@@ -8,7 +8,7 @@
  * Copyright (C) 2005
  *  Martin Paljak <martin@paljak.pri.ee>
  *
- * $Id: pcsclite.h.in 4732 2010-02-09 14:09:14Z rousseau $
+ * $Id: pcsclite.h.in 5288 2010-10-10 13:08:44Z rousseau $
  */
 
 /**
@@ -41,15 +41,12 @@ typedef struct
 {
 	const char *szReader;
 	void *pvUserData;
-	unsigned long dwCurrentState;
-	unsigned long dwEventState;
-	unsigned long cbAtr;
+	DWORD dwCurrentState;
+	DWORD dwEventState;
+	DWORD cbAtr;
 	unsigned char rgbAtr[MAX_ATR_SIZE];
 }
-SCARD_READERSTATE_A;
-
-typedef SCARD_READERSTATE_A SCARD_READERSTATE, *PSCARD_READERSTATE_A,
-	*LPSCARD_READERSTATE_A;
+SCARD_READERSTATE, *LPSCARD_READERSTATE;
 
 /** Protocol Control Information (PCI) */
 typedef struct
@@ -182,7 +179,7 @@ extern SCARD_IO_REQUEST g_rgSCardT0Pci, g_rgSCardT1Pci, g_rgSCardRawPci;
 #define INFINITE			0xFFFFFFFF	/**< Infinite timeout */
 #endif
 
-#define PCSCLITE_VERSION_NUMBER		"1.6.1"	/**< Current version */
+#define PCSCLITE_VERSION_NUMBER		"1.6.6"	/**< Current version */
 /** Maximum readers context (a slot is count as a reader) */
 #define PCSCLITE_MAX_READERS_CONTEXTS			16
 
@@ -203,7 +200,7 @@ extern SCARD_IO_REQUEST g_rgSCardT0Pci, g_rgSCardT1Pci, g_rgSCardRawPci;
 /*
  * Gets a stringified error response
  */
-char *pcsc_stringify_error(const long);
+char *pcsc_stringify_error(const LONG);
 
 #ifdef __cplusplus
 }
