@@ -3,10 +3,10 @@
  *
  * Copyright (C) 2000-2002
  *  David Corcoran <corcoran@linuxnet.com>
- * Copyright (C) 2002-2009
+ * Copyright (C) 2002-2010
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: eventhandler.c 4912 2010-05-03 09:30:25Z rousseau $
+ * $Id: eventhandler.c 5415 2010-11-23 08:30:14Z rousseau $
  */
 
 /**
@@ -39,7 +39,6 @@
 #include "winscard_svc.h"
 #include "simclist.h"
 
-READER_STATE readerStates[PCSCLITE_MAX_READERS_CONTEXTS];
 static list_t ClientsWaitingForEvent;	/**< list of client file descriptors */
 pthread_mutex_t ClientsWaitingForEvent_lock;	/**< lock for the above list */
 
@@ -50,7 +49,7 @@ LONG EHRegisterClientForEvent(int32_t filedes)
 	(void)pthread_mutex_lock(&ClientsWaitingForEvent_lock);
 
 	(void)list_append(&ClientsWaitingForEvent, &filedes);
-	
+
 	(void)pthread_mutex_unlock(&ClientsWaitingForEvent_lock);
 
 	return SCARD_S_SUCCESS;
@@ -58,7 +57,7 @@ LONG EHRegisterClientForEvent(int32_t filedes)
 
 /**
  * Try to unregisted a client
- * If no client is found then do no log an error
+ * If no client is found then do not log an error
  */
 LONG EHTryToUnregisterClientForEvent(int32_t filedes)
 {
@@ -68,9 +67,9 @@ LONG EHTryToUnregisterClientForEvent(int32_t filedes)
 	(void)pthread_mutex_lock(&ClientsWaitingForEvent_lock);
 
 	ret = list_delete(&ClientsWaitingForEvent, &filedes);
-	
+
 	(void)pthread_mutex_unlock(&ClientsWaitingForEvent_lock);
-	
+
 	if (ret < 0)
 		rv = SCARD_F_INTERNAL_ERROR;
 
@@ -83,7 +82,7 @@ LONG EHTryToUnregisterClientForEvent(int32_t filedes)
 LONG EHUnregisterClientForEvent(int32_t filedes)
 {
 	LONG rv = EHTryToUnregisterClientForEvent(filedes);
-	
+
 	if (rv < 0)
 		Log2(PCSC_LOG_ERROR, "Can't remove client: %d", filedes);
 
@@ -117,20 +116,6 @@ LONG EHSignalEventToClients(void)
 
 LONG EHInitializeEventStructures(void)
 {
-	int i;
-
-	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
-	{
-		/* Zero out each value in the struct */
-		memset(readerStates[i].readerName, 0, MAX_READERNAME);
-		memset(readerStates[i].cardAtr, 0, MAX_ATR_SIZE);
-		readerStates[i].readerID = 0;
-		readerStates[i].readerState = 0;
-		readerStates[i].readerSharing = 0;
-		readerStates[i].cardAtrLength = 0;
-		readerStates[i].cardProtocol = SCARD_PROTOCOL_UNDEFINED;
-	}
-
 	(void)list_init(&ClientsWaitingForEvent);
 
 	/* request to store copies, and provide the metric function */
@@ -150,12 +135,6 @@ LONG EHDestroyEventHandler(READER_CONTEXT * rContext)
 	DWORD dwGetSize;
 	UCHAR ucGetData[1];
 
-	if (NULL == rContext->readerState)
-	{
-		Log1(PCSC_LOG_ERROR, "Thread never started (reader init failed?)");
-		return SCARD_S_SUCCESS;
-	}
-
 	if ('\0' == rContext->readerState->readerName[0])
 	{
 		Log1(PCSC_LOG_INFO, "Thread already stomped.");
@@ -182,27 +161,28 @@ LONG EHDestroyEventHandler(READER_CONTEXT * rContext)
 	}
 	else
 #endif
-		Log1(PCSC_LOG_INFO, "Waiting polling thread");
+	{
+		/* ask to stop the "polling" thread */
+		RESPONSECODE (*fct)(DWORD) = NULL;
+
+		dwGetSize = sizeof(fct);
+		rv = IFDGetCapabilities(rContext, TAG_IFD_STOP_POLLING_THREAD,
+			&dwGetSize, (PUCHAR)&fct);
+
+		if ((IFD_SUCCESS == rv) && (dwGetSize == sizeof(fct)))
+		{
+			Log1(PCSC_LOG_INFO, "Request stoping of polling thread");
+			fct(rContext->slot);
+		}
+		else
+			Log1(PCSC_LOG_INFO, "Waiting polling thread");
+	}
 
 	/* wait for the thread to finish */
 	rv = pthread_join(rContext->pthThread, NULL);
 	if (rv)
 		Log2(PCSC_LOG_ERROR, "pthread_join failed: %s", strerror(rv));
 
-	/*
-	 * Zero out the public status struct to allow it to be recycled and
-	 * used again
-	 */
-	memset(rContext->readerState->readerName, 0,
-		sizeof(rContext->readerState->readerName));
-	memset(rContext->readerState->cardAtr, 0,
-		sizeof(rContext->readerState->cardAtr));
-	rContext->readerState->readerID = 0;
-	rContext->readerState->readerState = 0;
-	rContext->readerState->readerSharing = 0;
-	rContext->readerState->cardAtrLength = 0;
-	rContext->readerState->cardProtocol = SCARD_PROTOCOL_UNDEFINED;
-
 	/* Zero the thread */
 	rContext->pthThread = 0;
 
@@ -211,48 +191,19 @@ LONG EHDestroyEventHandler(READER_CONTEXT * rContext)
 	return SCARD_S_SUCCESS;
 }
 
-LONG EHSpawnEventHandler(READER_CONTEXT * rContext,
-	RESPONSECODE (*card_event)(DWORD))
+LONG EHSpawnEventHandler(READER_CONTEXT * rContext)
 {
 	LONG rv;
 	DWORD dwStatus = 0;
-	int i;
-	UCHAR ucAtr[MAX_ATR_SIZE];
-	DWORD dwAtrLen = 0;
 
-	rv = IFDStatusICC(rContext, &dwStatus, ucAtr, &dwAtrLen);
+	rv = IFDStatusICC(rContext, &dwStatus);
 	if (rv != SCARD_S_SUCCESS)
 	{
-		Log2(PCSC_LOG_ERROR, "Initial Check Failed on %s", rContext->lpcReader);
+		Log2(PCSC_LOG_ERROR, "Initial Check Failed on %s",
+			rContext->readerState->readerName);
 		return SCARD_F_UNKNOWN_ERROR;
 	}
 
-	/*
-	 * Find an empty reader slot and insert the new reader
-	 */
-	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
-	{
-		if (readerStates[i].readerID == 0)
-			break;
-	}
-
-	if (i == PCSCLITE_MAX_READERS_CONTEXTS)
-		return SCARD_F_INTERNAL_ERROR;
-
-	/*
-	 * Set all the attributes to this reader
-	 */
-	rContext->readerState = &readerStates[i];
-	(void)strlcpy(rContext->readerState->readerName, rContext->lpcReader,
-		sizeof(rContext->readerState->readerName));
-	memcpy(rContext->readerState->cardAtr, ucAtr, dwAtrLen);
-	rContext->readerState->readerID = i + 100;
-	rContext->readerState->readerState = dwStatus;
-	rContext->readerState->readerSharing = rContext->contexts;
-	rContext->readerState->cardAtrLength = dwAtrLen;
-	rContext->readerState->cardProtocol = SCARD_PROTOCOL_UNDEFINED;
-
-	rContext->pthCardEvent = card_event;
 	rv = ThreadCreate(&rContext->pthThread, 0,
 		(PCSCLITE_THREAD_FUNCTION( ))EHStatusHandlerThread, (LPVOID) rContext);
 	if (rv)
@@ -264,21 +215,12 @@ LONG EHSpawnEventHandler(READER_CONTEXT * rContext,
 		return SCARD_S_SUCCESS;
 }
 
-static void incrementEventCounter(struct pubReaderStatesList *readerState)
-{
-	int counter;
-
-	counter = (readerState -> readerState >> 16) & 0xFFFF;
-	counter++;
-	readerState -> readerState = (readerState -> readerState & 0xFFFF)
-		+ (counter << 16);
-}
-
 static void EHStatusHandlerThread(READER_CONTEXT * rContext)
 {
 	LONG rv;
-	LPCSTR lpcReader;
+	const char *readerName;
 	DWORD dwStatus;
+	uint32_t readerState;
 	int32_t readerSharing;
 	DWORD dwCurrentState;
 	DWORD dwAtrLen;
@@ -290,19 +232,21 @@ static void EHStatusHandlerThread(READER_CONTEXT * rContext)
 	readerSharing = 0;
 	dwCurrentState = 0;
 
-	lpcReader = rContext->lpcReader;
+	readerName = rContext->readerState->readerName;
 
-	dwAtrLen = rContext->readerState->cardAtrLength;
-	rv = IFDStatusICC(rContext, &dwStatus, rContext->readerState->cardAtr,
-		&dwAtrLen);
-	rContext->readerState->cardAtrLength = dwAtrLen;
+	rv = IFDStatusICC(rContext, &dwStatus);
 
 	if (dwStatus & SCARD_PRESENT)
 	{
-		dwAtrLen = MAX_ATR_SIZE;
+#ifdef DISABLE_AUTO_POWER_ON
+		rContext->readerState->cardAtrLength = 0;
+		rContext->readerState->cardProtocol = SCARD_PROTOCOL_UNDEFINED;
+		readerState = SCARD_PRESENT;
+		Log1(PCSC_LOG_INFO, "Skip card power on");
+#else
+		dwAtrLen = sizeof(rContext->readerState->cardAtr);
 		rv = IFDPowerICC(rContext, IFD_POWER_UP,
-			rContext->readerState->cardAtr,
-			&dwAtrLen);
+			rContext->readerState->cardAtr, &dwAtrLen);
 		rContext->readerState->cardAtrLength = dwAtrLen;
 
 		/* the protocol is unset after a power on */
@@ -310,13 +254,9 @@ static void EHStatusHandlerThread(READER_CONTEXT * rContext)
 
 		if (rv == IFD_SUCCESS)
 		{
-			dwStatus |= SCARD_PRESENT;
-			dwStatus &= ~SCARD_ABSENT;
-			dwStatus |= SCARD_POWERED;
-			dwStatus |= SCARD_NEGOTIABLE;
-			dwStatus &= ~SCARD_SPECIFIC;
-			dwStatus &= ~SCARD_SWALLOWED;
-			dwStatus &= ~SCARD_UNKNOWN;
+			readerState = SCARD_PRESENT | SCARD_POWERED | SCARD_NEGOTIABLE;
+			rContext->powerState = POWER_STATE_POWERED;
+			Log1(PCSC_LOG_DEBUG, "powerState: POWER_STATE_POWERED");
 
 			if (rContext->readerState->cardAtrLength > 0)
 			{
@@ -329,27 +269,18 @@ static void EHStatusHandlerThread(READER_CONTEXT * rContext)
 		}
 		else
 		{
-			dwStatus |= SCARD_PRESENT;
-			dwStatus &= ~SCARD_ABSENT;
-			dwStatus |= SCARD_SWALLOWED;
-			dwStatus &= ~SCARD_POWERED;
-			dwStatus &= ~SCARD_NEGOTIABLE;
-			dwStatus &= ~SCARD_SPECIFIC;
-			dwStatus &= ~SCARD_UNKNOWN;
+			readerState = SCARD_PRESENT | SCARD_SWALLOWED;
+			rContext->powerState = POWER_STATE_UNPOWERED;
+			Log1(PCSC_LOG_DEBUG, "powerState: POWER_STATE_UNPOWERED");
 			Log3(PCSC_LOG_ERROR, "Error powering up card: %d 0x%04X", rv, rv);
 		}
+#endif
 
 		dwCurrentState = SCARD_PRESENT;
 	}
 	else
 	{
-		dwStatus |= SCARD_ABSENT;
-		dwStatus &= ~SCARD_PRESENT;
-		dwStatus &= ~SCARD_POWERED;
-		dwStatus &= ~SCARD_NEGOTIABLE;
-		dwStatus &= ~SCARD_SPECIFIC;
-		dwStatus &= ~SCARD_SWALLOWED;
-		dwStatus &= ~SCARD_UNKNOWN;
+		readerState = SCARD_ABSENT;
 		rContext->readerState->cardAtrLength = 0;
 		rContext->readerState->cardProtocol = SCARD_PROTOCOL_UNDEFINED;
 
@@ -359,9 +290,8 @@ static void EHStatusHandlerThread(READER_CONTEXT * rContext)
 	/*
 	 * Set all the public attributes to this reader
 	 */
-	rContext->readerState->readerState = dwStatus;
-	rContext->readerState->readerSharing = readerSharing =
-		rContext->contexts;
+	rContext->readerState->readerState = readerState;
+	rContext->readerState->readerSharing = readerSharing = rContext->contexts;
 
 	(void)EHSignalEventToClients();
 
@@ -369,26 +299,16 @@ static void EHStatusHandlerThread(READER_CONTEXT * rContext)
 	{
 		dwStatus = 0;
 
-		dwAtrLen = rContext->readerState->cardAtrLength;
-		rv = IFDStatusICC(rContext, &dwStatus,
-			rContext->readerState->cardAtr,
-			&dwAtrLen);
-		rContext->readerState->cardAtrLength = dwAtrLen;
+		rv = IFDStatusICC(rContext, &dwStatus);
 
 		if (rv != SCARD_S_SUCCESS)
 		{
-			Log2(PCSC_LOG_ERROR, "Error communicating to: %s", lpcReader);
+			Log2(PCSC_LOG_ERROR, "Error communicating to: %s", readerName);
 
 			/*
 			 * Set error status on this reader while errors occur
 			 */
-			rContext->readerState->readerState &= ~SCARD_ABSENT;
-			rContext->readerState->readerState &= ~SCARD_PRESENT;
-			rContext->readerState->readerState &= ~SCARD_POWERED;
-			rContext->readerState->readerState &= ~SCARD_NEGOTIABLE;
-			rContext->readerState->readerState &= ~SCARD_SPECIFIC;
-			rContext->readerState->readerState &= ~SCARD_SWALLOWED;
-			rContext->readerState->readerState |= SCARD_UNKNOWN;
+			rContext->readerState->readerState = SCARD_UNKNOWN;
 			rContext->readerState->cardAtrLength = 0;
 			rContext->readerState->cardProtocol = SCARD_PROTOCOL_UNDEFINED;
 
@@ -405,7 +325,7 @@ static void EHStatusHandlerThread(READER_CONTEXT * rContext)
 				/*
 				 * Change the status structure
 				 */
-				Log2(PCSC_LOG_INFO, "Card Removed From %s", lpcReader);
+				Log2(PCSC_LOG_INFO, "Card Removed From %s", readerName);
 				/*
 				 * Notify the card has been removed
 				 */
@@ -413,16 +333,10 @@ static void EHStatusHandlerThread(READER_CONTEXT * rContext)
 
 				rContext->readerState->cardAtrLength = 0;
 				rContext->readerState->cardProtocol = SCARD_PROTOCOL_UNDEFINED;
-				rContext->readerState->readerState |= SCARD_ABSENT;
-				rContext->readerState->readerState &= ~SCARD_UNKNOWN;
-				rContext->readerState->readerState &= ~SCARD_PRESENT;
-				rContext->readerState->readerState &= ~SCARD_POWERED;
-				rContext->readerState->readerState &= ~SCARD_NEGOTIABLE;
-				rContext->readerState->readerState &= ~SCARD_SWALLOWED;
-				rContext->readerState->readerState &= ~SCARD_SPECIFIC;
+				rContext->readerState->readerState = SCARD_ABSENT;
 				dwCurrentState = SCARD_ABSENT;
 
-				incrementEventCounter(rContext->readerState);
+				rContext->readerState->eventCounter++;
 
 				(void)EHSignalEventToClients();
 			}
@@ -433,13 +347,22 @@ static void EHStatusHandlerThread(READER_CONTEXT * rContext)
 			if (dwCurrentState == SCARD_ABSENT ||
 				dwCurrentState == SCARD_UNKNOWN)
 			{
+#ifdef DISABLE_AUTO_POWER_ON
+				rContext->readerState->cardAtrLength = 0;
+				rContext->readerState->cardProtocol = SCARD_PROTOCOL_UNDEFINED;
+				rContext->readerState->readerState = SCARD_PRESENT;
+				rContext->powerState = POWER_STATE_UNPOWERED;
+				Log1(PCSC_LOG_DEBUG, "powerState: POWER_STATE_UNPOWERED");
+				readerState = SCARD_PRESENT;
+				rv = IFD_SUCCESS;
+				Log1(PCSC_LOG_INFO, "Skip card power on");
+#else
 				/*
 				 * Power and reset the card
 				 */
-				dwAtrLen = MAX_ATR_SIZE;
+				dwAtrLen = sizeof(rContext->readerState->cardAtr);
 				rv = IFDPowerICC(rContext, IFD_POWER_UP,
-					rContext->readerState->cardAtr,
-					&dwAtrLen);
+					rContext->readerState->cardAtr, &dwAtrLen);
 				rContext->readerState->cardAtrLength = dwAtrLen;
 
 				/* the protocol is unset after a power on */
@@ -447,31 +370,24 @@ static void EHStatusHandlerThread(READER_CONTEXT * rContext)
 
 				if (rv == IFD_SUCCESS)
 				{
-					rContext->readerState->readerState |= SCARD_PRESENT;
-					rContext->readerState->readerState &= ~SCARD_ABSENT;
-					rContext->readerState->readerState |= SCARD_POWERED;
-					rContext->readerState->readerState |= SCARD_NEGOTIABLE;
-					rContext->readerState->readerState &= ~SCARD_SPECIFIC;
-					rContext->readerState->readerState &= ~SCARD_UNKNOWN;
-					rContext->readerState->readerState &= ~SCARD_SWALLOWED;
+					rContext->readerState->readerState = SCARD_PRESENT | SCARD_POWERED | SCARD_NEGOTIABLE;
+					rContext->powerState = POWER_STATE_POWERED;
+					Log1(PCSC_LOG_DEBUG, "powerState: POWER_STATE_POWERED");
 				}
 				else
 				{
-					rContext->readerState->readerState |= SCARD_PRESENT;
-					rContext->readerState->readerState &= ~SCARD_ABSENT;
-					rContext->readerState->readerState |= SCARD_SWALLOWED;
-					rContext->readerState->readerState &= ~SCARD_POWERED;
-					rContext->readerState->readerState &= ~SCARD_NEGOTIABLE;
-					rContext->readerState->readerState &= ~SCARD_SPECIFIC;
-					rContext->readerState->readerState &= ~SCARD_UNKNOWN;
+					rContext->readerState->readerState = SCARD_PRESENT | SCARD_SWALLOWED;
+					rContext->powerState = POWER_STATE_UNPOWERED;
+					Log1(PCSC_LOG_DEBUG, "powerState: POWER_STATE_UNPOWERED");
 					rContext->readerState->cardAtrLength = 0;
 				}
+#endif
 
 				dwCurrentState = SCARD_PRESENT;
 
-				incrementEventCounter(rContext->readerState);
+				rContext->readerState->eventCounter++;
 
-				Log2(PCSC_LOG_INFO, "Card inserted into %s", lpcReader);
+				Log2(PCSC_LOG_INFO, "Card inserted into %s", readerName);
 
 				(void)EHSignalEventToClients();
 
@@ -504,14 +420,49 @@ static void EHStatusHandlerThread(READER_CONTEXT * rContext)
 		if (rContext->pthCardEvent)
 		{
 			int ret;
+			int timeout;
 
-			ret = rContext->pthCardEvent(rContext->slot);
+#ifndef DISABLE_ON_DEMAND_POWER_ON
+			if (POWER_STATE_POWERED == rContext->powerState)
+				/* The card is powered but not yet used */
+				timeout = PCSCLITE_POWER_OFF_GRACE_PERIOD;
+			else
+				/* The card is already in use or not used at all */
+#endif
+				timeout = PCSCLITE_STATUS_EVENT_TIMEOUT;
+
+			ret = rContext->pthCardEvent(rContext->slot, timeout);
 			if (IFD_NO_SUCH_DEVICE == ret)
 				(void)SYS_USleep(PCSCLITE_STATUS_POLL_RATE);
 		}
 		else
 			(void)SYS_USleep(PCSCLITE_STATUS_POLL_RATE);
 
+#ifndef DISABLE_ON_DEMAND_POWER_ON
+		/* the card is powered but not used */
+		(void)pthread_mutex_lock(&rContext->powerState_lock);
+		if (POWER_STATE_POWERED == rContext->powerState)
+		{
+			/* power down */
+			IFDPowerICC(rContext, IFD_POWER_DOWN, NULL, NULL);
+			rContext->powerState = POWER_STATE_UNPOWERED;
+			Log1(PCSC_LOG_DEBUG, "powerState: POWER_STATE_UNPOWERED");
+
+			/* the protocol is unset after a power down */
+			rContext->readerState->cardProtocol = SCARD_PROTOCOL_UNDEFINED;
+		}
+
+		/* the card was in use */
+		if (POWER_STATE_GRACE_PERIOD == rContext->powerState)
+		{
+			/* the next state should be UNPOWERED unless the
+			 * card is used again */
+			rContext->powerState = POWER_STATE_POWERED;
+			Log1(PCSC_LOG_DEBUG, "powerState: POWER_STATE_POWERED");
+		}
+		(void)pthread_mutex_unlock(&rContext->powerState_lock);
+#endif
+
 		if (rContext->hLockId == 0xFFFF)
 		{
 			/*
