@@ -5,10 +5,10 @@
  *  David Corcoran <corcoran@linuxnet.com>
  * Copyright (C) 2003-2004
  *  Damien Sauveron <damien.sauveron@labri.fr>
- * Copyright (C) 2002-2009
+ * Copyright (C) 2002-2010
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: ifdwrapper.c 4885 2010-04-18 12:17:56Z rousseau $
+ * $Id: ifdwrapper.c 5343 2010-10-22 11:52:22Z rousseau $
  */
 
 /**
@@ -32,6 +32,14 @@
 #include "sys_generic.h"
 #include "utils.h"
 
+#ifdef PCSCLITE_STATIC_DRIVER
+/* check that either IFDHANDLERv2 or IFDHANDLERv3 is
+ * defined */
+  #if ! (defined(IFDHANDLERv2) || defined(IFDHANDLERv3))
+  #error IFDHANDLER version not defined
+  #endif
+#endif
+
 /**
  * Set the protocol type selection (PTS).
  * This function sets the appropriate protocol to be used on the card.
@@ -43,28 +51,15 @@ LONG IFDSetPTS(READER_CONTEXT * rContext, DWORD dwProtocol, UCHAR ucFlags,
 	UCHAR ucValue[1];
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	RESPONSECODE(*IFD_set_protocol_parameters) (DWORD, UCHAR, UCHAR,
-		UCHAR, UCHAR) = NULL;
 	RESPONSECODE(*IFDH_set_protocol_parameters) (DWORD, DWORD, UCHAR,
 		UCHAR, UCHAR, UCHAR) = NULL;
 
-	if (rContext->version == IFD_HVERSION_1_0)
-	{
-		IFD_set_protocol_parameters = (RESPONSECODE(*)(DWORD, UCHAR, UCHAR,
-			UCHAR, UCHAR)) rContext->psFunctions.psFunctions_v1.pvfSetProtocolParameters;
+	IFDH_set_protocol_parameters = (RESPONSECODE(*)(DWORD, DWORD, UCHAR,
+		UCHAR, UCHAR, UCHAR))
+		rContext->psFunctions.psFunctions_v2.pvfSetProtocolParameters;
 
-		if (NULL == IFD_set_protocol_parameters)
-			return SCARD_E_UNSUPPORTED_FEATURE;
-	}
-	else
-	{
-		IFDH_set_protocol_parameters = (RESPONSECODE(*)(DWORD, DWORD, UCHAR,
-			UCHAR, UCHAR, UCHAR))
-			rContext->psFunctions.psFunctions_v2.pvfSetProtocolParameters;
-
-		if (NULL == IFDH_set_protocol_parameters)
-			return SCARD_E_UNSUPPORTED_FEATURE;
-	}
+	if (NULL == IFDH_set_protocol_parameters)
+		return SCARD_E_UNSUPPORTED_FEATURE;
 #endif
 
 	/*
@@ -77,30 +72,11 @@ LONG IFDSetPTS(READER_CONTEXT * rContext, DWORD dwProtocol, UCHAR ucFlags,
 	ucValue[0] = rContext->slot;
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	if (rContext->version == IFD_HVERSION_1_0)
-	{
-		ucValue[0] = rContext->slot;
-		(void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
-		rv = (*IFD_set_protocol_parameters) (dwProtocol,
-			ucFlags, ucPTS1, ucPTS2, ucPTS3);
-	}
-	else
-	{
-		rv = (*IFDH_set_protocol_parameters) (rContext->slot,
-			dwProtocol, ucFlags, ucPTS1, ucPTS2, ucPTS3);
-	}
-#else
-#ifdef IFDHANDLERv1
-	{
-		ucValue[0] = rContext->slot;
-		(void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
-		rv = IFD_Set_Protocol_Parameters(dwProtocol, ucFlags, ucPTS1,
-			ucPTS2, ucPTS3);
-	}
+	rv = (*IFDH_set_protocol_parameters) (rContext->slot,
+		dwProtocol, ucFlags, ucPTS1, ucPTS2, ucPTS3);
 #else
 	rv = IFDHSetProtocolParameters(rContext->slot, dwProtocol, ucFlags,
-		ucPTS1, ucPTS2, ucPTS3); 
-#endif
+		ucPTS1, ucPTS2, ucPTS3);
 #endif
 
 	return rv;
@@ -114,48 +90,38 @@ LONG IFDOpenIFD(READER_CONTEXT * rContext)
 	RESPONSECODE rv = 0;
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	RESPONSECODE(*IO_create_channel) (DWORD) = NULL;
 	RESPONSECODE(*IFDH_create_channel) (DWORD, DWORD) = NULL;
 	RESPONSECODE(*IFDH_create_channel_by_name) (DWORD, LPSTR) = NULL;
 
-	if (rContext->version == IFD_HVERSION_1_0)
-		IO_create_channel =
-			rContext->psFunctions.psFunctions_v1.pvfCreateChannel;
+	if (rContext->version == IFD_HVERSION_2_0)
+		IFDH_create_channel =
+			rContext->psFunctions.psFunctions_v2.pvfCreateChannel;
 	else
-		if (rContext->version == IFD_HVERSION_2_0)
-			IFDH_create_channel =
-				rContext->psFunctions.psFunctions_v2.pvfCreateChannel;
-		else
-		{
-			IFDH_create_channel =
-				rContext->psFunctions.psFunctions_v3.pvfCreateChannel;
-			IFDH_create_channel_by_name =
-				rContext->psFunctions.psFunctions_v3.pvfCreateChannelByName;
-		}
+	{
+		IFDH_create_channel =
+			rContext->psFunctions.psFunctions_v3.pvfCreateChannel;
+		IFDH_create_channel_by_name =
+			rContext->psFunctions.psFunctions_v3.pvfCreateChannelByName;
+	}
 #endif
 
 	/* LOCK THIS CODE REGION */
 	(void)pthread_mutex_lock(rContext->mMutex);
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	if (rContext->version == IFD_HVERSION_1_0)
-	{
-		rv = (*IO_create_channel) (rContext->port);
-	} else if (rContext->version == IFD_HVERSION_2_0)
+	if (rContext->version == IFD_HVERSION_2_0)
 	{
 		rv = (*IFDH_create_channel) (rContext->slot, rContext->port);
 	} else
 	{
 		/* use device name only if defined */
-		if (rContext->lpcDevice[0] != '\0')
-			rv = (*IFDH_create_channel_by_name) (rContext->slot, rContext->lpcDevice);
+		if (rContext->device[0] != '\0')
+			rv = (*IFDH_create_channel_by_name) (rContext->slot, rContext->device);
 		else
 			rv = (*IFDH_create_channel) (rContext->slot, rContext->port);
 	}
 #else
-#ifdef IFDHANDLERv1
-	rv = IO_Create_Channel(rContext->port);
-#elif IFDHANDLERv2
+#if defined(IFDHANDLERv2)
 	rv = IFDHCreateChannel(rContext->slot, rContext->port);
 #else
 	{
@@ -183,13 +149,9 @@ LONG IFDCloseIFD(READER_CONTEXT * rContext)
 	int repeat;
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	RESPONSECODE(*IO_close_channel) (void) = NULL;
 	RESPONSECODE(*IFDH_close_channel) (DWORD) = NULL;
 
-	if (rContext->version == IFD_HVERSION_1_0)
-		IO_close_channel = rContext->psFunctions.psFunctions_v1.pvfCloseChannel;
-	else
-		IFDH_close_channel = rContext->psFunctions.psFunctions_v2.pvfCloseChannel;
+	IFDH_close_channel = rContext->psFunctions.psFunctions_v2.pvfCloseChannel;
 #endif
 
 	/* TRY TO LOCK THIS CODE REGION */
@@ -208,17 +170,9 @@ again:
 	}
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	if (rContext->version == IFD_HVERSION_1_0)
-
-		rv = (*IO_close_channel) ();
-	else
-		rv = (*IFDH_close_channel) (rContext->slot);
-#else
-#ifdef IFDHANDLERv1
-	rv = IO_Close_Channel();
+	rv = (*IFDH_close_channel) (rContext->slot);
 #else
 	rv = IFDHCloseChannel(rContext->slot);
-#endif
 #endif
 
 	/* END OF LOCKED REGION */
@@ -236,13 +190,9 @@ LONG IFDSetCapabilities(READER_CONTEXT * rContext, DWORD dwTag,
 	RESPONSECODE rv = IFD_SUCCESS;
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	RESPONSECODE(*IFD_set_capabilities) (DWORD, PUCHAR) = NULL;
 	RESPONSECODE(*IFDH_set_capabilities) (DWORD, DWORD, DWORD, PUCHAR) = NULL;
 
-	if (rContext->version == IFD_HVERSION_1_0)
-		IFD_set_capabilities = rContext->psFunctions.psFunctions_v1.pvfSetCapabilities;
-	else
-		IFDH_set_capabilities = rContext->psFunctions.psFunctions_v2.pvfSetCapabilities;
+	IFDH_set_capabilities = rContext->psFunctions.psFunctions_v2.pvfSetCapabilities;
 #endif
 
 	/*
@@ -251,17 +201,10 @@ LONG IFDSetCapabilities(READER_CONTEXT * rContext, DWORD dwTag,
 	 */
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	if (rContext->version == IFD_HVERSION_1_0)
-		rv = (*IFD_set_capabilities) (dwTag, pucValue);
-	else
-		rv = (*IFDH_set_capabilities) (rContext->slot, dwTag,
+	rv = (*IFDH_set_capabilities) (rContext->slot, dwTag,
 			dwLength, pucValue);
-#else
-#ifdef IFDHANDLERv1
-	rv = IFD_Set_Capabilities(dwTag, pucValue);
 #else
 	rv = IFDHSetCapabilities(rContext->slot, dwTag, dwLength, pucValue);
-#endif
 #endif
 
 	return rv;
@@ -278,32 +221,19 @@ LONG IFDGetCapabilities(READER_CONTEXT * rContext, DWORD dwTag,
 	RESPONSECODE rv = IFD_SUCCESS;
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	RESPONSECODE(*IFD_get_capabilities) (DWORD, /*@out@*/ PUCHAR) = NULL;
 	RESPONSECODE(*IFDH_get_capabilities) (DWORD, DWORD, PDWORD, /*@out@*/ PUCHAR) = NULL;
 
-	if (rContext->version == IFD_HVERSION_1_0)
-		IFD_get_capabilities =
-			rContext->psFunctions.psFunctions_v1.pvfGetCapabilities;
-	else
-		IFDH_get_capabilities =
-			rContext->psFunctions.psFunctions_v2.pvfGetCapabilities;
+	IFDH_get_capabilities =
+		rContext->psFunctions.psFunctions_v2.pvfGetCapabilities;
 #endif
 
 	/* LOCK THIS CODE REGION */
 	(void)pthread_mutex_lock(rContext->mMutex);
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	if (rContext->version == IFD_HVERSION_1_0)
-		rv = (*IFD_get_capabilities) (dwTag, pucValue);
-	else
-		rv = (*IFDH_get_capabilities) (rContext->slot, dwTag,
-			pdwLength, pucValue);
-#else
-#ifdef IFDHANDLERv1
-	rv = IFD_Get_Capabilities(dwTag, pucValue);
+	rv = (*IFDH_get_capabilities) (rContext->slot, dwTag, pdwLength, pucValue);
 #else
 	rv = IFDHGetCapabilities(rContext->slot, dwTag, pdwLength, pucValue);
-#endif
 #endif
 
 	/* END OF LOCKED REGION */
@@ -325,9 +255,10 @@ LONG IFDPowerICC(READER_CONTEXT * rContext, DWORD dwAction,
 #endif
 	DWORD dwStatus;
 	UCHAR ucValue[1];
+	UCHAR dummyAtr[MAX_ATR_SIZE];
+	DWORD dummyAtrLen = sizeof(dummyAtr);
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	RESPONSECODE(*IFD_power_icc) (DWORD) = NULL;
 	RESPONSECODE(*IFDH_power_icc) (DWORD, DWORD, PUCHAR, PDWORD) = NULL;
 #endif
 
@@ -338,47 +269,31 @@ LONG IFDPowerICC(READER_CONTEXT * rContext, DWORD dwAction,
 	dwStatus = 0;
 	ucValue[0] = 0;
 
+	if (NULL == pucAtr)
+		pucAtr = dummyAtr;
+	if (NULL == pdwAtrLen)
+		pdwAtrLen = &dummyAtrLen;
+
 	/*
 	 * Check that the card is inserted first
 	 */
-	(void)IFDStatusICC(rContext, &dwStatus, pucAtr, pdwAtrLen);
+	(void)IFDStatusICC(rContext, &dwStatus);
 
 	if (dwStatus & SCARD_ABSENT)
 		return SCARD_W_REMOVED_CARD;
 #ifndef PCSCLITE_STATIC_DRIVER
-	if (rContext->version == IFD_HVERSION_1_0)
-		IFD_power_icc = rContext->psFunctions.psFunctions_v1.pvfPowerICC;
-	else
-		IFDH_power_icc = rContext->psFunctions.psFunctions_v2.pvfPowerICC;
+	IFDH_power_icc = rContext->psFunctions.psFunctions_v2.pvfPowerICC;
 #endif
 
 	/* LOCK THIS CODE REGION */
 	(void)pthread_mutex_lock(rContext->mMutex);
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	if (rContext->version == IFD_HVERSION_1_0)
-	{
-		ucValue[0] = rContext->slot;
-		(void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
-		rv = (*IFD_power_icc) (dwAction);
-	}
-	else
-	{
-		rv = (*IFDH_power_icc) (rContext->slot, dwAction,
-			pucAtr, pdwAtrLen);
+	rv = (*IFDH_power_icc) (rContext->slot, dwAction, pucAtr, pdwAtrLen);
 
-		ret = ATRDecodeAtr(&sSmartCard, pucAtr, *pdwAtrLen);
-	}
-#else
-#ifdef IFDHANDLERv1
-	{
-		ucValue[0] = rContext->slot;
-		(void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
-		rv = IFD_Power_ICC(dwAction);
-	}
+	ret = ATRDecodeAtr(&sSmartCard, pucAtr, *pdwAtrLen);
 #else
 	rv = IFDHPowerICC(rContext->slot, dwAction, pucAtr, pdwAtrLen);
-#endif
 #endif
 
 	/* END OF LOCKED REGION */
@@ -399,12 +314,6 @@ LONG IFDPowerICC(READER_CONTEXT * rContext, DWORD dwAction,
 		return SCARD_E_NOT_TRANSACTED;
 	}
 
-	/*
-	 * Get the ATR and it's length
-	 */
-	if (rContext->version == IFD_HVERSION_1_0)
-		(void)IFDStatusICC(rContext, &dwStatus, pucAtr, pdwAtrLen);
-
 	return rv;
 }
 
@@ -412,52 +321,24 @@ LONG IFDPowerICC(READER_CONTEXT * rContext, DWORD dwAction,
  * Provide statistical information about the IFD and ICC including insertions,
  * atr, powering status/etc.
  */
-LONG IFDStatusICC(READER_CONTEXT * rContext, PDWORD pdwStatus,
-	PUCHAR pucAtr, PDWORD pdwAtrLen)
+LONG IFDStatusICC(READER_CONTEXT * rContext, PDWORD pdwStatus)
 {
 	RESPONSECODE rv = IFD_SUCCESS;
-	DWORD dwTag = 0, dwCardStatus = 0;
-	SMARTCARD_EXTENSION sSmartCard;
-	UCHAR ucValue[1] = "\x00";
+	DWORD dwCardStatus = 0;
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	RESPONSECODE(*IFD_is_icc_present) (void) = NULL;
 	RESPONSECODE(*IFDH_icc_presence) (DWORD) = NULL;
-	RESPONSECODE(*IFD_get_capabilities) (DWORD, /*@out@*/ PUCHAR) = NULL;
 
-	if (rContext->version == IFD_HVERSION_1_0)
-	{
-		IFD_is_icc_present =
-			rContext->psFunctions.psFunctions_v1.pvfICCPresence;
-		IFD_get_capabilities =
-			rContext->psFunctions.psFunctions_v1.pvfGetCapabilities;
-	}
-	else
-		IFDH_icc_presence = rContext->psFunctions.psFunctions_v2.pvfICCPresence;
+	IFDH_icc_presence = rContext->psFunctions.psFunctions_v2.pvfICCPresence;
 #endif
 
 	/* LOCK THIS CODE REGION */
 	(void)pthread_mutex_lock(rContext->mMutex);
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	if (rContext->version == IFD_HVERSION_1_0)
-	{
-		ucValue[0] = rContext->slot;
-		(void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
-		rv = (*IFD_is_icc_present) ();
-	}
-	else
-		rv = (*IFDH_icc_presence) (rContext->slot);
-#else
-#ifdef IFDHANDLERv1
-	{
-		ucValue[0] = rContext->slot;
-		(void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
-		rv = IFD_Is_ICC_Present();
-	}
+	rv = (*IFDH_icc_presence) (rContext->slot);
 #else
 	rv = IFDHICCPresence(rContext->slot);
-#endif
 #endif
 
 	/* END OF LOCKED REGION */
@@ -482,64 +363,6 @@ LONG IFDStatusICC(READER_CONTEXT * rContext, PDWORD pdwStatus,
 			return SCARD_E_NOT_TRANSACTED;
 		}
 
-	/*
-	 * Now lets get the ATR and process it if IFD Handler version 1.0.
-	 * IFD Handler version 2.0 does this immediately after reset/power up
-	 * to conserve resources
-	 */
-
-	if (rContext->version == IFD_HVERSION_1_0)
-	{
-		if (rv == IFD_SUCCESS || rv == IFD_ICC_PRESENT)
-		{
-			short ret;
-
-			dwTag = TAG_IFD_ATR;
-
-			/* LOCK THIS CODE REGION */
-			(void)pthread_mutex_lock(rContext->mMutex);
-
-			ucValue[0] = rContext->slot;
-			(void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
-
-#ifndef PCSCLITE_STATIC_DRIVER
-			rv = (*IFD_get_capabilities) (dwTag, pucAtr);
-#else
-#ifdef IFDHANDLERv1
-			rv = IFD_Get_Capabilities(dwTag, pucAtr);
-#endif
-#endif
-
-			/* END OF LOCKED REGION */
-			(void)pthread_mutex_unlock(rContext->mMutex);
-
-			/*
-			 * FIX :: This is a temporary way to return the correct size
-			 * of the ATR since most of the drivers return MAX_ATR_SIZE
-			 */
-
-			ret = ATRDecodeAtr(&sSmartCard, pucAtr, MAX_ATR_SIZE);
-
-			/*
-			 * Might be a memory card without an ATR
-			 */
-			if (ret == 0)
-				*pdwAtrLen = 0;
-			else
-				*pdwAtrLen = sSmartCard.ATR.Length;
-		}
-		else
-		{
-			/*
-			 * No card is inserted - Atr length is 0
-			 */
-			*pdwAtrLen = 0;
-		}
-		/*
-		 * End of FIX
-		 */
-	}
-
 	*pdwStatus = dwCardStatus;
 
 	return SCARD_S_SUCCESS;
@@ -578,7 +401,7 @@ LONG IFDControl_v2(READER_CONTEXT * rContext, PUCHAR TxBuffer,
 #ifndef PCSCLITE_STATIC_DRIVER
 	rv = (*IFDH_control_v2) (rContext->slot, TxBuffer, TxLength,
 		RxBuffer, RxLength);
-#elif IFDHANDLERv2
+#elif defined(IFDHANDLERv2)
 	rv = IFDHControl(rContext->slot, TxBuffer, TxLength,
 		RxBuffer, RxLength);
 #endif
@@ -629,7 +452,7 @@ LONG IFDControl(READER_CONTEXT * rContext, DWORD ControlCode,
 #ifndef PCSCLITE_STATIC_DRIVER
 	rv = (*IFDH_control) (rContext->slot, ControlCode, TxBuffer,
 		TxLength, RxBuffer, RxLength, BytesReturned);
-#elif IFDHANDLERv3
+#elif defined(IFDHANDLERv3)
 	rv = IFDHControl(rContext->slot, ControlCode, TxBuffer,
 		TxLength, RxBuffer, RxLength, BytesReturned);
 #endif
@@ -673,8 +496,6 @@ LONG IFDTransmit(READER_CONTEXT * rContext, SCARD_IO_HEADER pioTxPci,
 	RESPONSECODE rv = IFD_SUCCESS;
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	RESPONSECODE(*IFD_transmit_to_icc) (SCARD_IO_HEADER, PUCHAR, DWORD,
-		/*@out@*/ PUCHAR, PDWORD, PSCARD_IO_HEADER) = NULL;
 	RESPONSECODE(*IFDH_transmit_to_icc) (DWORD, SCARD_IO_HEADER, PUCHAR,
 		DWORD, /*@out@*/ PUCHAR, PDWORD, PSCARD_IO_HEADER) = NULL;
 #endif
@@ -683,46 +504,20 @@ LONG IFDTransmit(READER_CONTEXT * rContext, SCARD_IO_HEADER pioTxPci,
 	DebugLogCategory(DEBUG_CATEGORY_APDU, pucTxBuffer, dwTxLength);
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	if (rContext->version == IFD_HVERSION_1_0)
-		IFD_transmit_to_icc =
-			rContext->psFunctions.psFunctions_v1.pvfTransmitToICC;
-	else
-		IFDH_transmit_to_icc =
-			rContext->psFunctions.psFunctions_v2.pvfTransmitToICC;
+	IFDH_transmit_to_icc =
+		rContext->psFunctions.psFunctions_v2.pvfTransmitToICC;
 #endif
 
 	/* LOCK THIS CODE REGION */
 	(void)pthread_mutex_lock(rContext->mMutex);
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	if (rContext->version == IFD_HVERSION_1_0)
-	{
-		UCHAR ucValue[1];
-
-		ucValue[0] = rContext->slot;
-		(void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
-		rv = (*IFD_transmit_to_icc) (pioTxPci, (LPBYTE) pucTxBuffer,
-			dwTxLength, pucRxBuffer, pdwRxLength, pioRxPci);
-	}
-	else
-		rv = (*IFDH_transmit_to_icc) (rContext->slot, pioTxPci,
-			(LPBYTE) pucTxBuffer, dwTxLength,
-			pucRxBuffer, pdwRxLength, pioRxPci);
-#else
-#ifdef IFDHANDLERv1
-	{
-		UCHAR ucValue[1];
-
-		ucValue[0] = rContext->slot;
-		(void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
-		rv = IFD_Transmit_to_ICC(pioTxPci, (LPBYTE) pucTxBuffer,
-			dwTxLength, pucRxBuffer, pdwRxLength, pioRxPci);
-	}
+	rv = (*IFDH_transmit_to_icc) (rContext->slot, pioTxPci, (LPBYTE)
+		pucTxBuffer, dwTxLength, pucRxBuffer, pdwRxLength, pioRxPci);
 #else
 	rv = IFDHTransmitToICC(rContext->slot, pioTxPci,
 		(LPBYTE) pucTxBuffer, dwTxLength,
 		pucRxBuffer, pdwRxLength, pioRxPci);
-#endif
 #endif
 
 	/* END OF LOCKED REGION */
