@@ -3,10 +3,10 @@
  *
  * Copyright (C) 1999-2002
  *  David Corcoran <corcoran@linuxnet.com>
- * Copyright (C) 2002-2009
+ * Copyright (C) 2002-2010
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: pcscdaemon.c 4969 2010-06-01 07:33:36Z rousseau $
+ * $Id: pcscdaemon.c 5343 2010-10-22 11:52:22Z rousseau $
  */
 
 /**
@@ -148,6 +148,7 @@ int main(int argc, char **argv)
 	int customMaxReaderHandles = 0;
 	int customMaxThreadCardHandles = 0;
 	int opt;
+	int limited_rights = FALSE;
 #ifdef HAVE_GETOPT_LONG
 	int option_index = 0;
 	static struct option long_options[] = {
@@ -195,6 +196,9 @@ int main(int argc, char **argv)
 	 */
 	DebugLogSetLogType(DEBUGLOG_SYSLOG_DEBUG);
 
+	/* if the process is setuid or setgid it may have some restrictions */
+	limited_rights = (getgid() != getegid()) && (getuid() != 0);
+
 	/*
 	 * Handle any command line arguments
 	 */
@@ -212,6 +216,11 @@ int main(int argc, char **argv)
 				break;
 #endif
 			case 'c':
+				if (limited_rights)
+				{
+					Log1(PCSC_LOG_CRITICAL, "Can't use a user specified config file");
+					return EXIT_FAILURE;
+				}
 				Log2(PCSC_LOG_INFO, "using new config file: %s", optarg);
 				newReaderConfig = optarg;
 				break;
@@ -245,6 +254,11 @@ int main(int argc, char **argv)
 				return EXIT_SUCCESS;
 
 			case 'a':
+				if (limited_rights)
+				{
+					Log1(PCSC_LOG_CRITICAL, "Can't log APDU (restricted)");
+					return EXIT_FAILURE;
+				}
 				(void)DebugLogSetCategory(DEBUG_CATEGORY_APDU);
 				break;
 
@@ -255,19 +269,25 @@ int main(int argc, char **argv)
 				break;
 
 			case 't':
-				customMaxThreadCounter = optarg ? atoi(optarg) : 0; 
+				customMaxThreadCounter = optarg ? atoi(optarg) : 0;
+				if (limited_rights && (customMaxThreadCounter < PCSC_MAX_CONTEXT_THREADS))
+					customMaxThreadCounter = PCSC_MAX_CONTEXT_THREADS;
 				Log2(PCSC_LOG_INFO, "setting customMaxThreadCounter to: %d",
 					customMaxThreadCounter);
 				break;
 
 			case 'r':
-				customMaxReaderHandles = optarg ? atoi(optarg) : 0; 
+				customMaxReaderHandles = optarg ? atoi(optarg) : 0;
+				if (limited_rights && (customMaxReaderHandles < PCSC_MAX_READER_HANDLES))
+					customMaxReaderHandles = PCSC_MAX_READER_HANDLES;
 				Log2(PCSC_LOG_INFO, "setting customMaxReaderHandles to: %d",
 					customMaxReaderHandles);
 				break;
 
 			case 's':
-				customMaxThreadCardHandles = optarg ? atoi(optarg) : 0; 
+				customMaxThreadCardHandles = optarg ? atoi(optarg) : 0;
+				if (limited_rights && (customMaxThreadCardHandles < PCSC_MAX_CONTEXT_CARD_HANDLES))
+					customMaxThreadCardHandles = PCSC_MAX_CONTEXT_CARD_HANDLES;
 				Log2(PCSC_LOG_INFO, "setting customMaxThreadCardHandles to: %d",
 					customMaxThreadCardHandles);
 				break;
@@ -370,6 +390,10 @@ int main(int argc, char **argv)
 	{
 		int pid;
 
+		/* create a new session so that Ctrl-C on the application will
+		 * not also quit pcscd */
+		setsid();
+
 		/* fork() so that pcscd always return in --auto-exit mode */
 		pid = fork();
 		if (-1 == pid )
@@ -474,7 +498,8 @@ int main(int argc, char **argv)
 		int f;
 		int mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
 
-		if ((f = open(PCSCLITE_RUN_PID, O_RDWR | O_CREAT, mode)) != -1)
+		f = open(PCSCLITE_RUN_PID, O_RDWR | O_CREAT, mode);
+		if (f != -1)
 		{
 			char pid[PID_ASCII_SIZE];
 
@@ -519,7 +544,7 @@ int main(int argc, char **argv)
 	}
 	else
 	{
-		rv = RFStartSerialReaders(PCSCLITE_READER_CONFIG);
+		rv = RFStartSerialReaders(PCSCLITE_CONFIG_DIR);
 		if (rv == -1)
 			at_exit();
 	}
@@ -572,7 +597,12 @@ int main(int argc, char **argv)
 
 	rv = HPRegisterForHotplugEvents();
 	if (rv)
+	{
+		Log1(PCSC_LOG_ERROR, "HPRegisterForHotplugEvents failed");
 		at_exit();
+	}
+
+	RFWaitForReaderInit();
 #endif
 
 	/*
@@ -663,6 +693,19 @@ static void signal_trap(int sig)
 			at_exit();
 		}
 	}
+	else
+	{
+		/* if pcscd do not want to die */
+		static int lives = 2;
+
+		lives--;
+		/* no live left. Something is blocking the normal death. */
+		if (0 == lives)
+		{
+			Log1(PCSC_LOG_INFO, "Forced suicide");
+			at_exit();
+		}
+	}
 }
 
 static void print_version (void)
