@@ -5,12 +5,12 @@
  *  David Corcoran <corcoran@linuxnet.com>
  * Copyright (C) 2003-2004
  *  Damien Sauveron <damien.sauveron@labri.fr>
- * Copyright (C) 2002-2009
+ * Copyright (C) 2002-2010
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  * Copyright (C) 2009
  *  Jean-Luc Giraud <jlgiraud@googlemail.com>
  *
- * $Id: readerfactory.c 4901 2010-04-29 07:26:36Z rousseau $
+ * $Id: readerfactory.c 5415 2010-11-23 08:30:14Z rousseau $
  */
 
 /**
@@ -49,6 +49,7 @@
 #endif
 
 static READER_CONTEXT * sReadersContexts[PCSCLITE_MAX_READERS_CONTEXTS];
+READER_STATE readerStates[PCSCLITE_MAX_READERS_CONTEXTS];
 static int maxReaderHandles = PCSC_MAX_READER_HANDLES;
 static DWORD dwNumReadersContexts = 0;
 #ifdef USE_SERIAL
@@ -86,15 +87,25 @@ LONG RFAllocateReaderSpace(unsigned int customMaxReaderHandles)
 	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
 	{
 		sReadersContexts[i] = malloc(sizeof(READER_CONTEXT));
-		(sReadersContexts[i])->vHandle = NULL;
-		(sReadersContexts[i])->readerState = NULL;
+		sReadersContexts[i]->vHandle = NULL;
+
+		/* Zero out each value in the struct */
+		memset(readerStates[i].readerName, 0, MAX_READERNAME);
+		memset(readerStates[i].cardAtr, 0, MAX_ATR_SIZE);
+		readerStates[i].readerState = 0;
+		readerStates[i].readerSharing = 0;
+		readerStates[i].cardAtrLength = READER_NOT_INITIALIZED;
+		readerStates[i].cardProtocol = SCARD_PROTOCOL_UNDEFINED;
+
+		sReadersContexts[i]->readerState = &readerStates[i];
 	}
 
 	/* Create public event structures */
 	return EHInitializeEventStructures();
 }
 
-LONG RFAddReader(LPSTR lpcReader, int port, LPSTR lpcLibrary, LPSTR lpcDevice)
+LONG RFAddReader(const char *readerName, int port, const char *library,
+	const char *device)
 {
 	DWORD dwContext = 0, dwGetSize;
 	UCHAR ucGetData[1], ucThread[1];
@@ -102,14 +113,14 @@ LONG RFAddReader(LPSTR lpcReader, int port, LPSTR lpcLibrary, LPSTR lpcDevice)
 	int i, j;
 	int lrv = 0;
 
-	if ((lpcReader == NULL) || (lpcLibrary == NULL) || (lpcDevice == NULL))
+	if ((readerName == NULL) || (library == NULL) || (device == NULL))
 		return SCARD_E_INVALID_VALUE;
 
 	/* Reader name too long? also count " 00 00"*/
-	if (strlen(lpcReader) > MAX_READERNAME - sizeof(" 00 00"))
+	if (strlen(readerName) > MAX_READERNAME - sizeof(" 00 00"))
 	{
 		Log3(PCSC_LOG_ERROR, "Reader name too long: %d chars instead of max %d",
-			strlen(lpcReader), MAX_READERNAME - sizeof(" 00 00"));
+			strlen(readerName), MAX_READERNAME - sizeof(" 00 00"));
 		return SCARD_E_INVALID_VALUE;
 	}
 
@@ -118,19 +129,20 @@ LONG RFAddReader(LPSTR lpcReader, int port, LPSTR lpcLibrary, LPSTR lpcDevice)
 	{
 		for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
 		{
-			if ((sReadersContexts[i])->vHandle != 0)
+			if (sReadersContexts[i]->vHandle != 0)
 			{
 				char lpcStripReader[MAX_READERNAME];
 				int tmplen;
 
 				/* get the reader name without the reader and slot numbers */
-				strncpy(lpcStripReader, (sReadersContexts[i])->lpcReader,
+				strncpy(lpcStripReader,
+					sReadersContexts[i]->readerState->readerName,
 					sizeof(lpcStripReader));
 				tmplen = strlen(lpcStripReader);
 				lpcStripReader[tmplen - 6] = 0;
 
-				if ((strcmp(lpcReader, lpcStripReader) == 0) &&
-					(port == (sReadersContexts[i])->port))
+				if ((strcmp(readerName, lpcStripReader) == 0) &&
+					(port == sReadersContexts[i]->port))
 				{
 					Log1(PCSC_LOG_ERROR, "Duplicate reader found.");
 					return SCARD_E_DUPLICATE_READER;
@@ -142,7 +154,7 @@ LONG RFAddReader(LPSTR lpcReader, int port, LPSTR lpcLibrary, LPSTR lpcDevice)
 	/* We must find an empty slot to put the reader structure */
 	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
 	{
-		if ((sReadersContexts[i])->vHandle == 0)
+		if (sReadersContexts[i]->vHandle == 0)
 		{
 			dwContext = i;
 			break;
@@ -156,77 +168,81 @@ LONG RFAddReader(LPSTR lpcReader, int port, LPSTR lpcLibrary, LPSTR lpcDevice)
 	}
 
 	/* Check and set the readername to see if it must be enumerated */
-	parentNode = RFSetReaderName(sReadersContexts[dwContext], lpcReader,
-		lpcLibrary, port, 0);
+	parentNode = RFSetReaderName(sReadersContexts[dwContext], readerName,
+		library, port, 0);
 	if (parentNode < -1)
 		return SCARD_E_NO_MEMORY;
 
-	sReadersContexts[dwContext]->lpcLibrary = strdup(lpcLibrary);
-	sReadersContexts[dwContext]->lpcDevice = strdup(lpcDevice);
-	(sReadersContexts[dwContext])->version = 0;
-	(sReadersContexts[dwContext])->port = port;
-	(sReadersContexts[dwContext])->mMutex = NULL;
-	(sReadersContexts[dwContext])->contexts = 0;
-	(sReadersContexts[dwContext])->pthThread = 0;
-	(sReadersContexts[dwContext])->hLockId = 0;
-	(sReadersContexts[dwContext])->LockCount = 0;
-	(sReadersContexts[dwContext])->vHandle = NULL;
-	(sReadersContexts[dwContext])->pFeeds = NULL;
-	(sReadersContexts[dwContext])->pMutex = NULL;
-	(sReadersContexts[dwContext])->dwIdentity =
+	sReadersContexts[dwContext]->library = strdup(library);
+	sReadersContexts[dwContext]->device = strdup(device);
+	sReadersContexts[dwContext]->version = 0;
+	sReadersContexts[dwContext]->port = port;
+	sReadersContexts[dwContext]->mMutex = NULL;
+	sReadersContexts[dwContext]->contexts = 0;
+	sReadersContexts[dwContext]->pthThread = 0;
+	sReadersContexts[dwContext]->hLockId = 0;
+	sReadersContexts[dwContext]->LockCount = 0;
+	sReadersContexts[dwContext]->vHandle = NULL;
+	sReadersContexts[dwContext]->pFeeds = NULL;
+	sReadersContexts[dwContext]->pMutex = NULL;
+	sReadersContexts[dwContext]->pthCardEvent = NULL;
+	sReadersContexts[dwContext]->dwIdentity =
 		(dwContext + 1) << IDENTITY_SHIFT;
-	(sReadersContexts[dwContext])->readerState = NULL;
 
-	lrv = list_init(&((sReadersContexts[dwContext])->handlesList));
+	lrv = list_init(&sReadersContexts[dwContext]->handlesList);
 	if (lrv < 0)
 	{
-		Log2(PCSC_LOG_CRITICAL, "list_init failed with return value: %X", lrv);
+		Log2(PCSC_LOG_CRITICAL, "list_init failed with return value: %d", lrv);
 		return SCARD_E_NO_MEMORY;
 	}
 
-	lrv = list_attributes_seeker(&((sReadersContexts[dwContext])->handlesList),
+	lrv = list_attributes_seeker(&sReadersContexts[dwContext]->handlesList,
 		RDR_CLIHANDLES_seeker);
 	if (lrv < 0)
 	{
 		Log2(PCSC_LOG_CRITICAL,
-			"list_attributes_seeker failed with return value: %X", lrv);
+			"list_attributes_seeker failed with return value: %d", lrv);
 		return SCARD_E_NO_MEMORY;
 	}
 
-	(void)pthread_mutex_init(&(sReadersContexts[dwContext])->handlesList_lock,
+	(void)pthread_mutex_init(&sReadersContexts[dwContext]->handlesList_lock,
 		NULL);
 
+	(void)pthread_mutex_init(&sReadersContexts[dwContext]->powerState_lock,
+		NULL);
+	sReadersContexts[dwContext]->powerState = POWER_STATE_UNPOWERED;
+
 	/* If a clone to this reader exists take some values from that clone */
 	if (parentNode >= 0 && parentNode < PCSCLITE_MAX_READERS_CONTEXTS)
 	{
-		(sReadersContexts[dwContext])->pFeeds =
-		  (sReadersContexts[parentNode])->pFeeds;
+		sReadersContexts[dwContext]->pFeeds =
+		  sReadersContexts[parentNode]->pFeeds;
 		*(sReadersContexts[dwContext])->pFeeds += 1;
-		(sReadersContexts[dwContext])->vHandle =
-		  (sReadersContexts[parentNode])->vHandle;
-		(sReadersContexts[dwContext])->mMutex =
-		  (sReadersContexts[parentNode])->mMutex;
-		(sReadersContexts[dwContext])->pMutex =
-		  (sReadersContexts[parentNode])->pMutex;
+		sReadersContexts[dwContext]->vHandle =
+		  sReadersContexts[parentNode]->vHandle;
+		sReadersContexts[dwContext]->mMutex =
+		  sReadersContexts[parentNode]->mMutex;
+		sReadersContexts[dwContext]->pMutex =
+		  sReadersContexts[parentNode]->pMutex;
 
 		/* Call on the parent driver to see if it is thread safe */
 		dwGetSize = sizeof(ucThread);
-		rv = IFDGetCapabilities((sReadersContexts[parentNode]),
+		rv = IFDGetCapabilities(sReadersContexts[parentNode],
 			TAG_IFD_THREAD_SAFE, &dwGetSize, ucThread);
 
 		if (rv == IFD_SUCCESS && dwGetSize == 1 && ucThread[0] == 1)
 		{
 			Log1(PCSC_LOG_INFO, "Driver is thread safe");
-			(sReadersContexts[dwContext])->mMutex = NULL;
-			(sReadersContexts[dwContext])->pMutex = NULL;
+			sReadersContexts[dwContext]->mMutex = NULL;
+			sReadersContexts[dwContext]->pMutex = NULL;
 		}
 		else
 			*(sReadersContexts[dwContext])->pMutex += 1;
 	}
 
-	if ((sReadersContexts[dwContext])->pFeeds == NULL)
+	if (sReadersContexts[dwContext]->pFeeds == NULL)
 	{
-		(sReadersContexts[dwContext])->pFeeds = malloc(sizeof(int));
+		sReadersContexts[dwContext]->pFeeds = malloc(sizeof(int));
 
 		/* Initialize pFeeds to 1, otherwise multiple
 		   cloned readers will cause pcscd to crash when
@@ -235,16 +251,16 @@ LONG RFAddReader(LPSTR lpcReader, int port, LPSTR lpcLibrary, LPSTR lpcDevice)
 		*(sReadersContexts[dwContext])->pFeeds = 1;
 	}
 
-	if ((sReadersContexts[dwContext])->mMutex == 0)
+	if (sReadersContexts[dwContext]->mMutex == 0)
 	{
-		(sReadersContexts[dwContext])->mMutex =
+		sReadersContexts[dwContext]->mMutex =
 			malloc(sizeof(pthread_mutex_t));
-		(void)pthread_mutex_init((sReadersContexts[dwContext])->mMutex, NULL);
+		(void)pthread_mutex_init(sReadersContexts[dwContext]->mMutex, NULL);
 	}
 
-	if ((sReadersContexts[dwContext])->pMutex == NULL)
+	if (sReadersContexts[dwContext]->pMutex == NULL)
 	{
-		(sReadersContexts[dwContext])->pMutex = malloc(sizeof(int));
+		sReadersContexts[dwContext]->pMutex = malloc(sizeof(int));
 		*(sReadersContexts[dwContext])->pMutex = 1;
 	}
 
@@ -254,32 +270,34 @@ LONG RFAddReader(LPSTR lpcReader, int port, LPSTR lpcLibrary, LPSTR lpcDevice)
 	if (rv != SCARD_S_SUCCESS)
 	{
 		/* Cannot connect to reader. Exit gracefully */
-		Log2(PCSC_LOG_ERROR, "%s init failed.", lpcReader);
-		(void)RFRemoveReader(lpcReader, port);
+		Log2(PCSC_LOG_ERROR, "%s init failed.", readerName);
+		(void)RFRemoveReader(readerName, port);
 		return rv;
 	}
 
 	/* asynchronous card movement?  */
 	{
-		RESPONSECODE (*fct)(DWORD) = NULL;
+		RESPONSECODE (*fct)(DWORD, int) = NULL;
 
 		dwGetSize = sizeof(fct);
 
-		rv = IFDGetCapabilities((sReadersContexts[dwContext]),
-			TAG_IFD_POLLING_THREAD, &dwGetSize, (PUCHAR)&fct);
+		rv = IFDGetCapabilities(sReadersContexts[dwContext],
+			TAG_IFD_POLLING_THREAD_WITH_TIMEOUT, &dwGetSize, (PUCHAR)&fct);
 		if ((rv != SCARD_S_SUCCESS) || (dwGetSize != sizeof(fct)))
 		{
-			fct = NULL;
 			Log1(PCSC_LOG_INFO, "Using the pcscd polling thread");
 		}
 		else
+		{
+			sReadersContexts[dwContext]->pthCardEvent = fct;
 			Log1(PCSC_LOG_INFO, "Using the reader polling thread");
+		}
 
-		rv = EHSpawnEventHandler(sReadersContexts[dwContext], fct);
+		rv = EHSpawnEventHandler(sReadersContexts[dwContext]);
 		if (rv != SCARD_S_SUCCESS)
 		{
-			Log2(PCSC_LOG_ERROR, "%s init failed.", lpcReader);
-			(void)RFRemoveReader(lpcReader, port);
+			Log2(PCSC_LOG_ERROR, "%s init failed.", readerName);
+			(void)RFRemoveReader(readerName, port);
 			return rv;
 		}
 	}
@@ -308,12 +326,12 @@ LONG RFAddReader(LPSTR lpcReader, int port, LPSTR lpcLibrary, LPSTR lpcDevice)
 	{
 		char *tmpReader = NULL;
 		DWORD dwContextB = 0;
-		RESPONSECODE (*fct)(DWORD) = NULL;
+		RESPONSECODE (*fct)(DWORD, int) = NULL;
 
 		/* We must find an empty spot to put the reader structure */
 		for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
 		{
-			if ((sReadersContexts[i])->vHandle == 0)
+			if (sReadersContexts[i]->vHandle == 0)
 			{
 				dwContextB = i;
 				break;
@@ -323,30 +341,31 @@ LONG RFAddReader(LPSTR lpcReader, int port, LPSTR lpcLibrary, LPSTR lpcDevice)
 		if (i == PCSCLITE_MAX_READERS_CONTEXTS)
 		{
 			/* No more slot left return */
-			rv = RFRemoveReader(lpcReader, port);
+			rv = RFRemoveReader(readerName, port);
 			return SCARD_E_NO_MEMORY;
 		}
 
 		/* Copy the previous reader name and increment the slot number */
-		tmpReader = sReadersContexts[dwContextB]->lpcReader;
-		(void)strlcpy(tmpReader, sReadersContexts[dwContext]->lpcReader,
-			sizeof(sReadersContexts[dwContextB]->lpcReader));
+		tmpReader = sReadersContexts[dwContextB]->readerState->readerName;
+		(void)strlcpy(tmpReader,
+			sReadersContexts[dwContext]->readerState->readerName,
+			sizeof(sReadersContexts[dwContextB]->readerState->readerName));
 		sprintf(tmpReader + strlen(tmpReader) - 2, "%02X", j);
 
-		sReadersContexts[dwContextB]->lpcLibrary =
-			sReadersContexts[dwContext]->lpcLibrary;
-		sReadersContexts[dwContextB]->lpcDevice =
-			sReadersContexts[dwContext]->lpcDevice;
-		(sReadersContexts[dwContextB])->version =
-		  (sReadersContexts[dwContext])->version;
-		(sReadersContexts[dwContextB])->port =
-		  (sReadersContexts[dwContext])->port;
-		(sReadersContexts[dwContextB])->vHandle =
-		  (sReadersContexts[dwContext])->vHandle;
-		(sReadersContexts[dwContextB])->mMutex =
-		  (sReadersContexts[dwContext])->mMutex;
-		(sReadersContexts[dwContextB])->pMutex =
-		  (sReadersContexts[dwContext])->pMutex;
+		sReadersContexts[dwContextB]->library =
+			sReadersContexts[dwContext]->library;
+		sReadersContexts[dwContextB]->device =
+			sReadersContexts[dwContext]->device;
+		sReadersContexts[dwContextB]->version =
+		  sReadersContexts[dwContext]->version;
+		sReadersContexts[dwContextB]->port =
+		  sReadersContexts[dwContext]->port;
+		sReadersContexts[dwContextB]->vHandle =
+		  sReadersContexts[dwContext]->vHandle;
+		sReadersContexts[dwContextB]->mMutex =
+		  sReadersContexts[dwContext]->mMutex;
+		sReadersContexts[dwContextB]->pMutex =
+		  sReadersContexts[dwContext]->pMutex;
 		sReadersContexts[dwContextB]->slot =
 			sReadersContexts[dwContext]->slot + j;
 
@@ -354,36 +373,38 @@ LONG RFAddReader(LPSTR lpcReader, int port, LPSTR lpcLibrary, LPSTR lpcDevice)
 		 * Added by Dave - slots did not have a pFeeds
 		 * parameter so it was by luck they were working
 		 */
-		(sReadersContexts[dwContextB])->pFeeds =
-		  (sReadersContexts[dwContext])->pFeeds;
+		sReadersContexts[dwContextB]->pFeeds =
+		  sReadersContexts[dwContext]->pFeeds;
 
 		/* Added by Dave for multiple slots */
 		*(sReadersContexts[dwContextB])->pFeeds += 1;
 
-		(sReadersContexts[dwContextB])->contexts = 0;
-		(sReadersContexts[dwContextB])->hLockId = 0;
-		(sReadersContexts[dwContextB])->LockCount = 0;
-		(sReadersContexts[dwContextB])->readerState = NULL;
-		(sReadersContexts[dwContextB])->dwIdentity =
+		sReadersContexts[dwContextB]->contexts = 0;
+		sReadersContexts[dwContextB]->hLockId = 0;
+		sReadersContexts[dwContextB]->LockCount = 0;
+		sReadersContexts[dwContextB]->dwIdentity =
 			(dwContextB + 1) << IDENTITY_SHIFT;
 
-		lrv = list_init(&((sReadersContexts[dwContextB])->handlesList));
+		lrv = list_init(&sReadersContexts[dwContextB]->handlesList);
 		if (lrv < 0)
 		{
-			Log2(PCSC_LOG_CRITICAL, "list_init failed with return value: %X", lrv);
+			Log2(PCSC_LOG_CRITICAL, "list_init failed with return value: %d", lrv);
 			return SCARD_E_NO_MEMORY;
 		}
 
-		lrv = list_attributes_seeker(&((sReadersContexts[dwContextB])->handlesList),
+		lrv = list_attributes_seeker(&sReadersContexts[dwContextB]->handlesList,
 			RDR_CLIHANDLES_seeker);
 		if (lrv < 0)
 		{
 			Log2(PCSC_LOG_CRITICAL,
-					"list_attributes_seeker failed with return value: %X", lrv);
+					"list_attributes_seeker failed with return value: %d", lrv);
 			return SCARD_E_NO_MEMORY;
 		}
 
-		(void)pthread_mutex_init(&(sReadersContexts[dwContextB])->handlesList_lock, NULL);
+		(void)pthread_mutex_init(&sReadersContexts[dwContextB]->handlesList_lock, NULL);
+		(void)pthread_mutex_init(&sReadersContexts[dwContextB]->powerState_lock,
+			NULL);
+		sReadersContexts[dwContextB]->powerState = POWER_STATE_UNPOWERED;
 
 		/* Call on the parent driver to see if the slots are thread safe */
 		dwGetSize = sizeof(ucThread);
@@ -392,12 +413,12 @@ LONG RFAddReader(LPSTR lpcReader, int port, LPSTR lpcLibrary, LPSTR lpcDevice)
 
 		if (rv == IFD_SUCCESS && dwGetSize == 1 && ucThread[0] == 1)
 		{
-			(sReadersContexts[dwContextB])->mMutex =
+			sReadersContexts[dwContextB]->mMutex =
 				malloc(sizeof(pthread_mutex_t));
-			(void)pthread_mutex_init((sReadersContexts[dwContextB])->mMutex,
+			(void)pthread_mutex_init(sReadersContexts[dwContextB]->mMutex,
 				NULL);
 
-			(sReadersContexts[dwContextB])->pMutex = malloc(sizeof(int));
+			sReadersContexts[dwContextB]->pMutex = malloc(sizeof(int));
 			*(sReadersContexts[dwContextB])->pMutex = 1;
 		}
 		else
@@ -409,7 +430,7 @@ LONG RFAddReader(LPSTR lpcReader, int port, LPSTR lpcLibrary, LPSTR lpcDevice)
 		if (rv != SCARD_S_SUCCESS)
 		{
 			/* Cannot connect to slot. Exit gracefully */
-			(void)RFRemoveReader(lpcReader, port);
+			(void)RFRemoveReader(readerName, port);
 			return rv;
 		}
 
@@ -417,20 +438,22 @@ LONG RFAddReader(LPSTR lpcReader, int port, LPSTR lpcLibrary, LPSTR lpcDevice)
 		dwGetSize = sizeof(fct);
 
 		rv = IFDGetCapabilities((sReadersContexts[dwContextB]),
-				TAG_IFD_POLLING_THREAD, &dwGetSize, (PUCHAR)&fct);
+			TAG_IFD_POLLING_THREAD_WITH_TIMEOUT, &dwGetSize, (PUCHAR)&fct);
 		if ((rv != SCARD_S_SUCCESS) || (dwGetSize != sizeof(fct)))
 		{
-			fct = NULL;
 			Log1(PCSC_LOG_INFO, "Using the pcscd polling thread");
 		}
 		else
+		{
+			sReadersContexts[dwContextB]->pthCardEvent = fct;
 			Log1(PCSC_LOG_INFO, "Using the reader polling thread");
+		}
 
-		rv = EHSpawnEventHandler(sReadersContexts[dwContextB], fct);
+		rv = EHSpawnEventHandler(sReadersContexts[dwContextB]);
 		if (rv != SCARD_S_SUCCESS)
 		{
-			Log2(PCSC_LOG_ERROR, "%s init failed.", lpcReader);
-			(void)RFRemoveReader(lpcReader, port);
+			Log2(PCSC_LOG_ERROR, "%s init failed.", readerName);
+			(void)RFRemoveReader(readerName, port);
 			return rv;
 		}
 	}
@@ -438,20 +461,20 @@ LONG RFAddReader(LPSTR lpcReader, int port, LPSTR lpcLibrary, LPSTR lpcDevice)
 	return SCARD_S_SUCCESS;
 }
 
-LONG RFRemoveReader(LPSTR lpcReader, int port)
+LONG RFRemoveReader(const char *readerName, int port)
 {
 	LONG rv;
 	READER_CONTEXT * sContext;
 
-	if (lpcReader == 0)
+	if (readerName == NULL)
 		return SCARD_E_INVALID_VALUE;
 
 	while (SCARD_S_SUCCESS ==
-		RFReaderInfoNamePort(port, lpcReader, &sContext))
+		RFReaderInfoNamePort(port, readerName, &sContext))
 	{
-
 		/* Try to destroy the thread */
-		rv = EHDestroyEventHandler(sContext);
+		if (sContext -> pthThread)
+			(void)EHDestroyEventHandler(sContext);
 
 		rv = RFUnInitializeReader(sContext);
 		if (rv != SCARD_S_SUCCESS)
@@ -472,8 +495,8 @@ LONG RFRemoveReader(LPSTR lpcReader, int port)
 		{
 			(void)pthread_mutex_destroy(sContext->mMutex);
 			free(sContext->mMutex);
-			free(sContext->lpcLibrary);
-			free(sContext->lpcDevice);
+			free(sContext->library);
+			free(sContext->device);
 			free(sContext->pMutex);
 			sContext->pMutex = NULL;
 		}
@@ -496,7 +519,6 @@ LONG RFRemoveReader(LPSTR lpcReader, int port)
 		sContext->LockCount = 0;
 		sContext->vHandle = NULL;
 		sContext->dwIdentity = 0;
-		sContext->readerState = NULL;
 
 		(void)pthread_mutex_lock(&sContext->handlesList_lock);
 		while (list_size(&(sContext->handlesList)) != 0)
@@ -508,7 +530,7 @@ LONG RFRemoveReader(LPSTR lpcReader, int port)
 			lrv = list_delete_at(&(sContext->handlesList), 0);
 			if (lrv < 0)
 				Log2(PCSC_LOG_CRITICAL,
-					"list_delete_at failed with return value: %X", lrv);
+					"list_delete_at failed with return value: %d", lrv);
 
 			free(currentHandle);
 		}
@@ -523,8 +545,8 @@ LONG RFRemoveReader(LPSTR lpcReader, int port)
 	return SCARD_S_SUCCESS;
 }
 
-LONG RFSetReaderName(READER_CONTEXT * rContext, LPSTR readerName,
-	LPSTR libraryName, int port, DWORD slot)
+LONG RFSetReaderName(READER_CONTEXT * rContext, const char *readerName,
+	const char *libraryName, int port, DWORD slot)
 {
 	LONG parent = -1;	/* reader number of the parent of the clone */
 	DWORD valueLength;
@@ -541,16 +563,16 @@ LONG RFSetReaderName(READER_CONTEXT * rContext, LPSTR readerName,
 	{
 		for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
 		{
-			if ((sReadersContexts[i])->vHandle != 0)
+			if (sReadersContexts[i]->vHandle != 0)
 			{
-				if (strcmp((sReadersContexts[i])->lpcLibrary, libraryName) == 0)
+				if (strcmp(sReadersContexts[i]->library, libraryName) == 0)
 				{
 					UCHAR tagValue[1];
 					LONG ret;
 
 					/* Ask the driver if it supports multiple channels */
 					valueLength = sizeof(tagValue);
-					ret = IFDGetCapabilities((sReadersContexts[i]),
+					ret = IFDGetCapabilities(sReadersContexts[i],
 						TAG_IFD_SIMULTANEOUS_ACCESS,
 						&valueLength, tagValue);
 
@@ -565,12 +587,12 @@ LONG RFSetReaderName(READER_CONTEXT * rContext, LPSTR readerName,
 						supportedChannels = 1;
 
 					/* Check to see if it is a hotplug reader and different */
-					if (((((sReadersContexts[i])->port & 0xFFFF0000) ==
+					if ((((sReadersContexts[i]->port & 0xFFFF0000) ==
 							PCSCLITE_HP_BASE_PORT)
-						&& ((sReadersContexts[i])->port != port))
+						&& (sReadersContexts[i]->port != port))
 						|| (supportedChannels > 1))
 					{
-						char *lpcReader = sReadersContexts[i]->lpcReader;
+						const char *reader = sReadersContexts[i]->readerState->readerName;
 
 						/*
 						 * tells the caller who the parent of this
@@ -584,7 +606,7 @@ LONG RFSetReaderName(READER_CONTEXT * rContext, LPSTR readerName,
 						 * hotplug then we must look for others and
 						 * enumerate the readername
 						 */
-						currentDigit = strtol(lpcReader + strlen(lpcReader) - 5, NULL, 16);
+						currentDigit = strtol(reader + strlen(reader) - 5, NULL, 16);
 
 						/* This spot is taken */
 						usedDigits[currentDigit] = TRUE;
@@ -622,7 +644,8 @@ LONG RFSetReaderName(READER_CONTEXT * rContext, LPSTR readerName,
 		}
 	}
 
-	snprintf(rContext->lpcReader, sizeof(rContext->lpcReader), "%s %02X %02lX",
+	snprintf(rContext->readerState->readerName,
+		sizeof(rContext->readerState->readerName), "%s %02X %02lX",
 		readerName, i, slot);
 
 	/* Set the slot in 0xDDDDCCCC */
@@ -631,18 +654,19 @@ LONG RFSetReaderName(READER_CONTEXT * rContext, LPSTR readerName,
 	return parent;
 }
 
-LONG RFReaderInfo(LPSTR lpcReader, READER_CONTEXT ** sReader)
+LONG RFReaderInfo(const char *readerName, READER_CONTEXT ** sReader)
 {
 	int i;
 
-	if (lpcReader == 0)
+	if (readerName == NULL)
 		return SCARD_E_UNKNOWN_READER;
 
 	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
 	{
-		if ((sReadersContexts[i])->vHandle != 0)
+		if (sReadersContexts[i]->vHandle != 0)
 		{
-			if (strcmp(lpcReader, (sReadersContexts[i])->lpcReader) == 0)
+			if (strcmp(readerName,
+				sReadersContexts[i]->readerState->readerName) == 0)
 			{
 				*sReader = sReadersContexts[i];
 				return SCARD_S_SUCCESS;
@@ -653,7 +677,7 @@ LONG RFReaderInfo(LPSTR lpcReader, READER_CONTEXT ** sReader)
 	return SCARD_E_UNKNOWN_READER;
 }
 
-LONG RFReaderInfoNamePort(int port, LPSTR lpcReader,
+LONG RFReaderInfoNamePort(int port, const char *readerName,
 	READER_CONTEXT * * sReader)
 {
 	char lpcStripReader[MAX_READERNAME];
@@ -661,17 +685,18 @@ LONG RFReaderInfoNamePort(int port, LPSTR lpcReader,
 
 	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
 	{
-		if ((sReadersContexts[i])->vHandle != 0)
+		if (sReadersContexts[i]->vHandle != 0)
 		{
 			int tmplen;
 
-			strncpy(lpcStripReader, (sReadersContexts[i])->lpcReader,
+			strncpy(lpcStripReader,
+				sReadersContexts[i]->readerState->readerName,
 				sizeof(lpcStripReader));
 			tmplen = strlen(lpcStripReader);
 			lpcStripReader[tmplen - 6] = 0;
 
-			if ((strcmp(lpcReader, lpcStripReader) == 0) &&
-				(port == (sReadersContexts[i])->port))
+			if ((strcmp(readerName, lpcStripReader) == 0) &&
+				(port == sReadersContexts[i]->port))
 			{
 				*sReader = sReadersContexts[i];
 				return SCARD_S_SUCCESS;
@@ -692,7 +717,7 @@ LONG RFReaderInfoById(DWORD dwIdentity, READER_CONTEXT * * sReader)
 
 	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
 	{
-		if (dwIdentity == (sReadersContexts[i])->dwIdentity)
+		if (dwIdentity == sReadersContexts[i]->dwIdentity)
 		{
 			*sReader = sReadersContexts[i];
 			return SCARD_S_SUCCESS;
@@ -707,102 +732,49 @@ LONG RFLoadReader(READER_CONTEXT * rContext)
 	if (rContext->vHandle != 0)
 	{
 		Log2(PCSC_LOG_INFO, "Reusing already loaded driver for %s",
-			rContext->lpcLibrary);
+			rContext->library);
 		/* Another reader exists with this library loaded */
 		return SCARD_S_SUCCESS;
 	}
 
-	return DYN_LoadLibrary(&rContext->vHandle, rContext->lpcLibrary);
+	return DYN_LoadLibrary(&rContext->vHandle, rContext->library);
 }
 
 LONG RFBindFunctions(READER_CONTEXT * rContext)
 {
-	int rv1, rv2, rv3;
+	int rv;
 	void *f;
 
-	/*
-	 * Use this function as a dummy to determine the IFD Handler version
-	 * type  1.0/2.0/3.0.  Suppress error messaging since it can't be 1.0,
-	 * 2.0 and 3.0.
-	 */
-
-	DebugLogSuppress(DEBUGLOG_IGNORE_ENTRIES);
-
-	rv1 = DYN_GetAddress(rContext->vHandle, &f, "IO_Create_Channel");
-	rv2 = DYN_GetAddress(rContext->vHandle, &f, "IFDHCreateChannel");
-	rv3 = DYN_GetAddress(rContext->vHandle, &f, "IFDHCreateChannelByName");
-
-	DebugLogSuppress(DEBUGLOG_LOG_ENTRIES);
-
-	if (rv1 != SCARD_S_SUCCESS && rv2 != SCARD_S_SUCCESS && rv3 != SCARD_S_SUCCESS)
-	{
-		/* Neither version of the IFD Handler was found - exit */
-		Log1(PCSC_LOG_CRITICAL, "IFDHandler functions missing");
-
-		return SCARD_F_UNKNOWN_ERROR;
-	} else if (rv1 == SCARD_S_SUCCESS)
-	{
-		/* Ifd Handler 1.0 found */
-		rContext->version = IFD_HVERSION_1_0;
-	} else if (rv3 == SCARD_S_SUCCESS)
+	rv = DYN_GetAddress(rContext->vHandle, &f, "IFDHCreateChannelByName");
+	if (SCARD_S_SUCCESS == rv)
 	{
 		/* Ifd Handler 3.0 found */
 		rContext->version = IFD_HVERSION_3_0;
 	}
 	else
 	{
-		/* Ifd Handler 2.0 found */
-		rContext->version = IFD_HVERSION_2_0;
-	}
-
-	/* The following binds version 1.0 of the IFD Handler specs */
-	if (rContext->version == IFD_HVERSION_1_0)
-	{
-		Log1(PCSC_LOG_INFO, "Loading IFD Handler 1.0");
-
-#define GET_ADDRESS_OPTIONALv1(field, function, code) \
-{ \
-	void *f1 = NULL; \
-	DWORD rv = DYN_GetAddress(rContext->vHandle, &f1, "IFD_" #function); \
-	if (SCARD_S_SUCCESS != rv) \
-	{ \
-		code \
-	} \
-	rContext->psFunctions.psFunctions_v1.pvf ## field = f1; \
-}
-
-#define GET_ADDRESSv1(field, function) \
-	GET_ADDRESS_OPTIONALv1(field, function, \
-		Log1(PCSC_LOG_CRITICAL, "IFDHandler functions missing: " #function ); \
-		return(rv); )
-
-		(void)DYN_GetAddress(rContext->vHandle, &f, "IO_Create_Channel");
-		rContext->psFunctions.psFunctions_v1.pvfCreateChannel = f;
-
-		if (SCARD_S_SUCCESS != DYN_GetAddress(rContext->vHandle, &f,
-			"IO_Close_Channel"))
+		rv = DYN_GetAddress(rContext->vHandle, &f, "IFDHCreateChannel");
+		if (SCARD_S_SUCCESS == rv)
 		{
+			/* Ifd Handler 2.0 found */
+			rContext->version = IFD_HVERSION_2_0;
+		}
+		else
+		{
+			/* Neither version of the IFD Handler was found - exit */
 			Log1(PCSC_LOG_CRITICAL, "IFDHandler functions missing");
 			return SCARD_F_UNKNOWN_ERROR;
 		}
-		rContext->psFunctions.psFunctions_v1.pvfCloseChannel = f;
-
-		GET_ADDRESSv1(GetCapabilities, Get_Capabilities)
-		GET_ADDRESSv1(SetCapabilities, Set_Capabilities)
-		GET_ADDRESSv1(PowerICC, Power_ICC)
-		GET_ADDRESSv1(TransmitToICC, Transmit_to_ICC)
-		GET_ADDRESSv1(ICCPresence, Is_ICC_Present)
-
-		GET_ADDRESS_OPTIONALv1(SetProtocolParameters, Set_Protocol_Parameters, )
 	}
-	else if (rContext->version == IFD_HVERSION_2_0)
+
+	if (rContext->version == IFD_HVERSION_2_0)
 	{
 		/* The following binds version 2.0 of the IFD Handler specs */
 #define GET_ADDRESS_OPTIONALv2(s, code) \
 { \
 	void *f1 = NULL; \
-	DWORD rv = DYN_GetAddress(rContext->vHandle, &f1, "IFDH" #s); \
-	if (SCARD_S_SUCCESS != rv) \
+	int rvl = DYN_GetAddress(rContext->vHandle, &f1, "IFDH" #s); \
+	if (SCARD_S_SUCCESS != rvl) \
 	{ \
 		code \
 	} \
@@ -833,8 +805,8 @@ LONG RFBindFunctions(READER_CONTEXT * rContext)
 #define GET_ADDRESS_OPTIONALv3(s, code) \
 { \
 	void *f1 = NULL; \
-	DWORD rv = DYN_GetAddress(rContext->vHandle, &f1, "IFDH" #s); \
-	if (SCARD_S_SUCCESS != rv) \
+	int rvl = DYN_GetAddress(rContext->vHandle, &f1, "IFDH" #s); \
+	if (SCARD_S_SUCCESS != rvl) \
 	{ \
 		code \
 	} \
@@ -892,31 +864,20 @@ LONG RFUnloadReader(READER_CONTEXT * rContext)
 	return SCARD_S_SUCCESS;
 }
 
-LONG RFCheckSharing(SCARDHANDLE hCard)
+LONG RFCheckSharing(SCARDHANDLE hCard, READER_CONTEXT * rContext)
 {
-	LONG rv;
-	READER_CONTEXT * rContext = NULL;
-
-	rv = RFReaderInfoById(hCard, &rContext);
-
-	if (rv != SCARD_S_SUCCESS)
-		return rv;
-
 	if (rContext->hLockId == 0 || rContext->hLockId == hCard)
 		return SCARD_S_SUCCESS;
 	else
 		return SCARD_E_SHARING_VIOLATION;
 }
 
-LONG RFLockSharing(SCARDHANDLE hCard)
+LONG RFLockSharing(SCARDHANDLE hCard, READER_CONTEXT * rContext)
 {
-	READER_CONTEXT * rContext = NULL;
 	LONG rv;
 
-	(void)RFReaderInfoById(hCard, &rContext);
-
 	(void)pthread_mutex_lock(&LockMutex);
-	rv = RFCheckSharing(hCard);
+	rv = RFCheckSharing(hCard, rContext);
 	if (SCARD_S_SUCCESS == rv)
 	{
 		rContext->LockCount += 1;
@@ -927,17 +888,12 @@ LONG RFLockSharing(SCARDHANDLE hCard)
 	return rv;
 }
 
-LONG RFUnlockSharing(SCARDHANDLE hCard)
+LONG RFUnlockSharing(SCARDHANDLE hCard, READER_CONTEXT * rContext)
 {
-	READER_CONTEXT * rContext = NULL;
 	LONG rv;
 
-	rv = RFReaderInfoById(hCard, &rContext);
-	if (rv != SCARD_S_SUCCESS)
-		return rv;
-
 	(void)pthread_mutex_lock(&LockMutex);
-	rv = RFCheckSharing(hCard);
+	rv = RFCheckSharing(hCard, rContext);
 	if (SCARD_S_SUCCESS == rv)
 	{
 		if (rContext->LockCount > 0)
@@ -950,17 +906,12 @@ LONG RFUnlockSharing(SCARDHANDLE hCard)
 	return rv;
 }
 
-LONG RFUnlockAllSharing(SCARDHANDLE hCard)
+LONG RFUnlockAllSharing(SCARDHANDLE hCard, READER_CONTEXT * rContext)
 {
-	READER_CONTEXT * rContext = NULL;
 	LONG rv;
 
-	rv = RFReaderInfoById(hCard, &rContext);
-	if (rv != SCARD_S_SUCCESS)
-		return rv;
-
 	(void)pthread_mutex_lock(&LockMutex);
-	rv = RFCheckSharing(hCard);
+	rv = RFCheckSharing(hCard, rContext);
 	if (SCARD_S_SUCCESS == rv)
 	{
 		rContext->LockCount = 0;
@@ -977,14 +928,14 @@ LONG RFInitializeReader(READER_CONTEXT * rContext)
 
 	/* Spawn the event handler thread */
 	Log3(PCSC_LOG_INFO, "Attempting startup of %s using %s",
-		rContext->lpcReader, rContext->lpcLibrary);
+		rContext->readerState->readerName, rContext->library);
 
 #ifndef PCSCLITE_STATIC_DRIVER
 	/* loads the library */
 	rv = RFLoadReader(rContext);
 	if (rv != SCARD_S_SUCCESS)
 	{
-		Log2(PCSC_LOG_ERROR, "RFLoadReader failed: %X", rv);
+		Log2(PCSC_LOG_ERROR, "RFLoadReader failed: 0x%X", rv);
 		return rv;
 	}
 
@@ -993,7 +944,7 @@ LONG RFInitializeReader(READER_CONTEXT * rContext)
 
 	if (rv != SCARD_S_SUCCESS)
 	{
-		Log2(PCSC_LOG_ERROR, "RFBindFunctions failed: %X", rv);
+		Log2(PCSC_LOG_ERROR, "RFBindFunctions failed: 0x%X", rv);
 		(void)RFUnloadReader(rContext);
 		return rv;
 	}
@@ -1007,10 +958,15 @@ LONG RFInitializeReader(READER_CONTEXT * rContext)
 
 	if (rv != IFD_SUCCESS)
 	{
-		Log3(PCSC_LOG_CRITICAL, "Open Port %X Failed (%s)",
-			rContext->port, rContext->lpcDevice);
-		(void)RFUnBindFunctions(rContext);
-		(void)RFUnloadReader(rContext);
+		Log3(PCSC_LOG_CRITICAL, "Open Port 0x%X Failed (%s)",
+			rContext->port, rContext->device);
+
+		/* the reader was not started correctly */
+		rContext->slot = -1;
+
+		/* IFDOpenIFD() failed */
+		rContext->slot = -1;
+
 		if (IFD_NO_SUCH_DEVICE == rv)
 			return SCARD_E_UNKNOWN_READER;
 		else
@@ -1023,20 +979,28 @@ LONG RFInitializeReader(READER_CONTEXT * rContext)
 LONG RFUnInitializeReader(READER_CONTEXT * rContext)
 {
 	Log2(PCSC_LOG_INFO, "Attempting shutdown of %s.",
-		rContext->lpcReader);
+		rContext->readerState->readerName);
 
-	/* Close the port, unbind the functions, and unload the library */
+	/* Do not close a reader if IFDOpenIFD() failed in RFInitializeReader() */
+	if (rContext->slot != -1)
+		(void)IFDCloseIFD(rContext);
 
-	/*
-	 * If the reader is getting uninitialized then it is being unplugged
-	 * so I can't send a IFDPowerICC call to it
-	 *
-	 * IFDPowerICC( rContext, IFD_POWER_DOWN, Atr, &AtrLen );
-	 */
-	(void)IFDCloseIFD(rContext);
 	(void)RFUnBindFunctions(rContext);
 	(void)RFUnloadReader(rContext);
 
+	/*
+	 * Zero out the public status struct to allow it to be recycled and
+	 * used again
+	 */
+	memset(rContext->readerState->readerName, 0,
+		sizeof(rContext->readerState->readerName));
+	memset(rContext->readerState->cardAtr, 0,
+		sizeof(rContext->readerState->cardAtr));
+	rContext->readerState->readerState = 0;
+	rContext->readerState->readerSharing = 0;
+	rContext->readerState->cardAtrLength = READER_NOT_INITIALIZED;
+	rContext->readerState->cardProtocol = SCARD_PROTOCOL_UNDEFINED;
+
 	return SCARD_S_SUCCESS;
 }
 
@@ -1055,12 +1019,12 @@ SCARDHANDLE RFCreateReaderHandle(READER_CONTEXT * rContext)
 again:
 	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
 	{
-		if ((sReadersContexts[i])->vHandle != 0)
+		if (sReadersContexts[i]->vHandle != 0)
 		{
 			RDR_CLIHANDLES *currentHandle;
-			list_t * l = &((sReadersContexts[i])->handlesList);
+			list_t * l = &sReadersContexts[i]->handlesList;
 
-			(void)pthread_mutex_lock(&(sReadersContexts[i])->handlesList_lock);
+			(void)pthread_mutex_lock(&sReadersContexts[i]->handlesList_lock);
 			list_iterator_start(l);
 			while (list_iterator_hasnext(l))
 			{
@@ -1071,12 +1035,12 @@ again:
 					/* Get a new handle and loop again */
 					randHandle = SYS_RandomInt(10, 65000);
 					list_iterator_stop(l);
-					(void)pthread_mutex_unlock(&(sReadersContexts[i])->handlesList_lock);
+					(void)pthread_mutex_unlock(&sReadersContexts[i]->handlesList_lock);
 					goto again;
 				}
 			}
 			list_iterator_stop(l);
-			(void)pthread_mutex_unlock(&(sReadersContexts[i])->handlesList_lock);
+			(void)pthread_mutex_unlock(&sReadersContexts[i]->handlesList_lock);
 		}
 	}
 
@@ -1091,13 +1055,13 @@ LONG RFFindReaderHandle(SCARDHANDLE hCard)
 
 	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
 	{
-		if ((sReadersContexts[i])->vHandle != 0)
+		if (sReadersContexts[i]->vHandle != 0)
 		{
 			RDR_CLIHANDLES * currentHandle;
-			(void)pthread_mutex_lock(&(sReadersContexts[i])->handlesList_lock);
-			currentHandle = list_seek(&((sReadersContexts[i])->handlesList),
+			(void)pthread_mutex_lock(&sReadersContexts[i]->handlesList_lock);
+			currentHandle = list_seek(&sReadersContexts[i]->handlesList,
 				&hCard);
-			(void)pthread_mutex_unlock(&(sReadersContexts[i])->handlesList_lock);
+			(void)pthread_mutex_unlock(&sReadersContexts[i]->handlesList_lock);
 			if (currentHandle != NULL)
 				return SCARD_S_SUCCESS;
 		}
@@ -1146,7 +1110,7 @@ LONG RFAddReaderHandle(READER_CONTEXT * rContext, SCARDHANDLE hCard)
 	if (lrv < 0)
 	{
 		free(newHandle);
-		Log2(PCSC_LOG_CRITICAL, "list_append failed with return value: %X",
+		Log2(PCSC_LOG_CRITICAL, "list_append failed with return value: %d",
 			lrv);
 		rv = SCARD_E_NO_MEMORY;
 	}
@@ -1173,7 +1137,7 @@ LONG RFRemoveReaderHandle(READER_CONTEXT * rContext, SCARDHANDLE hCard)
 	lrv = list_delete(&(rContext->handlesList), currentHandle);
 	if (lrv < 0)
 		Log2(PCSC_LOG_CRITICAL,
-			"list_delete failed with return value: %X", lrv);
+			"list_delete failed with return value: %d", lrv);
 
 	free(currentHandle);
 
@@ -1228,7 +1192,7 @@ LONG RFCheckReaderEventState(READER_CONTEXT * rContext, SCARDHANDLE hCard)
 	if (NULL == currentHandle)
 	{
 		/* Not Found */
-		Log2(PCSC_LOG_CRITICAL, "list_seek failed for hCard %X", hCard);
+		Log2(PCSC_LOG_CRITICAL, "list_seek failed for hCard 0x%X", hCard);
 		return SCARD_E_INVALID_HANDLE;
 	}
 
@@ -1274,8 +1238,7 @@ LONG RFClearReaderEventState(READER_CONTEXT * rContext, SCARDHANDLE hCard)
 
 LONG RFCheckReaderStatus(READER_CONTEXT * rContext)
 {
-	if ((rContext->readerState == NULL)
-		|| (rContext->readerState->readerState & SCARD_UNKNOWN))
+	if (rContext->readerState->readerState & SCARD_UNKNOWN)
 		return SCARD_E_READER_UNAVAILABLE;
 	else
 		return SCARD_S_SUCCESS;
@@ -1294,9 +1257,10 @@ void RFCleanupReaders(void)
 			char lpcStripReader[MAX_READERNAME];
 
 			Log2(PCSC_LOG_INFO, "Stopping reader: %s",
-				sReadersContexts[i]->lpcReader);
+				sReadersContexts[i]->readerState->readerName);
 
-			strncpy(lpcStripReader, (sReadersContexts[i])->lpcReader,
+			strncpy(lpcStripReader,
+				sReadersContexts[i]->readerState->readerName,
 				sizeof(lpcStripReader));
 			/* strip the 6 last char ' 00 00' */
 			lpcStripReader[strlen(lpcStripReader) - 6] = '\0';
@@ -1309,10 +1273,44 @@ void RFCleanupReaders(void)
 	}
 }
 
+/**
+ * Wait until all connected readers have a chance to power up a possibly
+ * inserted card.
+ */
+#ifdef USE_USB
+void RFWaitForReaderInit(void)
+{
+	int i, need_to_wait;
+
+	do
+	{
+		need_to_wait = FALSE;
+		for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
+		{
+			/* reader is present */
+			if (sReadersContexts[i]->vHandle != NULL)
+			{
+				/* but card state is not yet available */
+				if (READER_NOT_INITIALIZED
+					== sReadersContexts[i]->readerState->cardAtrLength)
+				{
+					Log2(PCSC_LOG_DEBUG, "Waiting init for reader: %s",
+						sReadersContexts[i]->readerState->readerName);
+					need_to_wait = TRUE;
+				}
+			}
+		}
+
+		if (need_to_wait)
+			SYS_USleep(10*1000); /* 10 ms */
+	} while (need_to_wait);
+}
+#endif
+
 #ifdef USE_SERIAL
 int RFStartSerialReaders(const char *readerconf)
 {
-	SerialReader *reader_list;
+	SerialReader *reader_list = NULL;
 	int i, rv;
 
 	/* remember the configuration filename for RFReCheckReaderConf() */
@@ -1352,7 +1350,7 @@ int RFStartSerialReaders(const char *readerconf)
 
 void RFReCheckReaderConf(void)
 {
-	SerialReader *reader_list;
+	SerialReader *reader_list = NULL;
 	int i, crc;
 
 	(void)DBGetReaderListDir(ConfigFile, &reader_list);
@@ -1401,7 +1399,8 @@ void RFReCheckReaderConf(void)
 				int tmplen;
 
 				/* get the reader name without the reader and slot numbers */
-				strncpy(lpcStripReader, sReadersContexts[i]->lpcReader,
+				strncpy(lpcStripReader,
+					sReadersContexts[i]->readerState->readerName,
 					sizeof(lpcStripReader));
 				tmplen = strlen(lpcStripReader);
 				lpcStripReader[tmplen - 6] = 0;
@@ -1409,15 +1408,14 @@ void RFReCheckReaderConf(void)
 				if ((strcmp(reader_list[i].pcFriendlyname, lpcStripReader) == 0)
 					&& (reader_list[r].channelId == sReadersContexts[i]->port))
 				{
-					DWORD dwStatus = 0, dwAtrLen = 0;
-					UCHAR ucAtr[MAX_ATR_SIZE];
+					DWORD dwStatus = 0;
 
 					/* the reader was already started */
 					present = TRUE;
 
 					/* verify the reader is still connected */
-					if (IFDStatusICC(sReadersContexts[r], &dwStatus, ucAtr,
-						&dwAtrLen) != SCARD_S_SUCCESS)
+					if (IFDStatusICC(sReadersContexts[r], &dwStatus)
+						!= SCARD_S_SUCCESS)
 					{
 						Log2(PCSC_LOG_INFO, "Reader %s disappeared",
 							reader_list[i].pcFriendlyname);
@@ -1444,61 +1442,3 @@ void RFReCheckReaderConf(void)
 }
 #endif
 
-#if 0
-void RFSuspendAllReaders(void)
-{
-	int i;
-
-	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
-	{
-		if ((sReadersContexts[i])->vHandle != 0)
-		{
-			(void)EHDestroyEventHandler(sReadersContexts[i]);
-			(void)IFDCloseIFD(sReadersContexts[i]);
-		}
-	}
-}
-
-void RFAwakeAllReaders(void)
-{
-	LONG rv = IFD_SUCCESS;
-	int i;
-	int initFlag;
-
-	initFlag = 0;
-
-	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
-	{
-		/* If the library is loaded and the event handler is not running */
-		if ( ((sReadersContexts[i])->vHandle != 0) &&
-			((sReadersContexts[i])->pthThread == 0) )
-		{
-			int j;
-
-			for (j=0; j < i; j++)
-			{
-				if (((sReadersContexts[j])->vHandle == (sReadersContexts[i])->vHandle)&&
-					((sReadersContexts[j])->port == (sReadersContexts[i])->port))
-				{
-					initFlag = 1;
-				}
-			}
-
-			if (initFlag == 0)
-				rv = IFDOpenIFD(sReadersContexts[i]);
-			else
-				initFlag = 0;
-
-			if (rv != IFD_SUCCESS)
-			{
-				Log3(PCSC_LOG_ERROR, "Open Port %X Failed (%s)",
-					(sReadersContexts[i])->port, (sReadersContexts[i])->lpcDevice);
-			}
-
-			(void)EHSpawnEventHandler(sReadersContexts[i], NULL);
-			(void)RFSetReaderEventState(sReadersContexts[i], SCARD_RESET);
-		}
-	}
-}
-#endif
-
