@@ -3,10 +3,10 @@
  *
  * Copyright (C) 1999
  *  David Corcoran <corcoran@linuxnet.com>
- * Copyright (C) 2002-2009
+ * Copyright (C) 2002-2010
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: readerfactory.h 4910 2010-05-03 09:24:54Z rousseau $
+ * $Id: readerfactory.h 5434 2010-12-08 14:13:21Z rousseau $
  */
 
 /**
@@ -24,11 +24,6 @@
 #include "pcscd.h"
 #include "simclist.h"
 
-#ifdef __cplusplus
-extern "C"
-{
-#endif
-
 	typedef struct
 	{
 		char *pcFriendlyname;	/**< FRIENDLYNAME */
@@ -37,22 +32,6 @@ extern "C"
 		int channelId;		/**< CHANNELID */
 	} SerialReader;
 
-	struct FctMap_V1
-	{
-		RESPONSECODE (*pvfCreateChannel)(DWORD);
-		RESPONSECODE (*pvfCloseChannel)(void);
-		RESPONSECODE (*pvfGetCapabilities)(DWORD, PUCHAR);
-		RESPONSECODE (*pvfSetCapabilities)(DWORD, PUCHAR);
-		RESPONSECODE (*pvfSetProtocolParameters)(DWORD, UCHAR, UCHAR, UCHAR,
-			UCHAR);
-		RESPONSECODE (*pvfPowerICC)(DWORD);
-		RESPONSECODE (*pvfTransmitToICC)(SCARD_IO_HEADER, PUCHAR, DWORD,
-			PUCHAR, PDWORD, PSCARD_IO_HEADER);
-		RESPONSECODE (*pvfICCPresence)(void);
-	};
-
-	typedef struct FctMap_V1 FCT_MAP_V1;
-
 	struct FctMap_V2
 	{
 		/* shared with API 3.0 */
@@ -105,18 +84,16 @@ extern "C"
 
 	struct ReaderContext
 	{
-		char lpcReader[MAX_READERNAME];	/**< Reader Name */
-		char *lpcLibrary;	/**< Library Path */
-		char *lpcDevice;	/**< Device Name */
+		char *library;	/**< Library Path */
+		char *device;	/**< Device Name */
 		pthread_t pthThread;	/**< Event polling thread */
-		RESPONSECODE (*pthCardEvent)(DWORD);	/**< Card Event sync */
+		RESPONSECODE (*pthCardEvent)(DWORD, int);	/**< Card Event sync */
 		pthread_mutex_t *mMutex;	/**< Mutex for this connection */
 		list_t handlesList;
 		pthread_mutex_t handlesList_lock;	/**< lock for the above list */
                                          /**< Structure of connected handles */
 		union
 		{
-			FCT_MAP_V1 psFunctions_v1;	/**< API V1.0 */
 			FCT_MAP_V2 psFunctions_v2;	/**< API V2.0 */
 			FCT_MAP_V3 psFunctions_v3;	/**< API V3.0 */
 		} psFunctions;	/**< driver functions */
@@ -131,6 +108,8 @@ extern "C"
 		int32_t contexts;		/**< Number of open contexts */
 		int * pFeeds;			/**< Number of shared client to lib */
 		int * pMutex;			/**< Number of client to mutex */
+		int powerState;			/**< auto power off state */
+		pthread_mutex_t powerState_lock;	/**< powerState mutex */
 
 		struct pubReaderStatesList *readerState; /**< link to the reader state */
 		/* we can't use READER_CONTEXT * here since eventhandler.h can't be
@@ -140,16 +119,16 @@ extern "C"
 	typedef struct ReaderContext READER_CONTEXT;
 
 	LONG RFAllocateReaderSpace(unsigned int);
-	LONG RFAddReader(LPSTR, int, LPSTR, LPSTR);
-	LONG RFRemoveReader(LPSTR, int);
-	LONG RFSetReaderName(READER_CONTEXT *, LPSTR, LPSTR, int, DWORD);
-	LONG RFReaderInfo(LPSTR, /*@out@*/ struct ReaderContext **);
-	LONG RFReaderInfoNamePort(int, LPSTR, /*@out@*/ struct ReaderContext **);
+	LONG RFAddReader(const char *, int, const char *, const char *);
+	LONG RFRemoveReader(const char *, int);
+	LONG RFSetReaderName(READER_CONTEXT *, const char *, const char *, int, DWORD);
+	LONG RFReaderInfo(const char *, /*@out@*/ struct ReaderContext **);
+	LONG RFReaderInfoNamePort(int, const char *, /*@out@*/ struct ReaderContext **);
 	LONG RFReaderInfoById(DWORD, /*@out@*/ struct ReaderContext **);
-	LONG RFCheckSharing(SCARDHANDLE);
-	LONG RFLockSharing(SCARDHANDLE);
-	LONG RFUnlockSharing(SCARDHANDLE);
-	LONG RFUnlockAllSharing(SCARDHANDLE);
+	LONG RFCheckSharing(SCARDHANDLE, READER_CONTEXT *);
+	LONG RFLockSharing(SCARDHANDLE, READER_CONTEXT *);
+	LONG RFUnlockSharing(SCARDHANDLE, READER_CONTEXT *);
+	LONG RFUnlockAllSharing(SCARDHANDLE, READER_CONTEXT *);
 	LONG RFLoadReader(READER_CONTEXT *);
 	LONG RFBindFunctions(READER_CONTEXT *);
 	LONG RFUnBindFunctions(READER_CONTEXT *);
@@ -166,13 +145,8 @@ extern "C"
 	LONG RFClearReaderEventState(READER_CONTEXT *, SCARDHANDLE);
 	LONG RFCheckReaderStatus(READER_CONTEXT *);
 	void RFCleanupReaders(void);
+	void RFWaitForReaderInit(void);
 	int RFStartSerialReaders(const char *readerconf);
 	void RFReCheckReaderConf(void);
-	void RFSuspendAllReaders(void);
-	void RFAwakeAllReaders(void);
-
-#ifdef __cplusplus
-}
-#endif
 
 #endif
