@@ -3,10 +3,10 @@
  *
  * Copyright (C) 1999-2004
  *  David Corcoran <corcoran@linuxnet.com>
- * Copyright (C) 2002-2009
+ * Copyright (C) 2002-2010
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: winscard.c 4964 2010-05-27 10:12:34Z rousseau $
+ * $Id: winscard.c 5415 2010-11-23 08:30:14Z rousseau $
  */
 
 /**
@@ -170,36 +170,11 @@ static void profile_end(const char *f, int line)
 /** used for backward compatibility */
 #define SCARD_PROTOCOL_ANY_OLD	 0x1000
 
-/**
- * @brief Creates an Application Context for a client.
- *
- * This must be the first function called in a PC/SC application.
- *
- * @param[in] dwScope Scope of the establishment.
- * This can either be a local or remote connection.
- * - SCARD_SCOPE_USER - Not used.
- * - SCARD_SCOPE_TERMINAL - Not used.
- * - SCARD_SCOPE_GLOBAL - Not used.
- * - SCARD_SCOPE_SYSTEM - Services on the local machine.
- * @param[in] pvReserved1 Reserved for future use. Can be used for remote connection.
- * @param[in] pvReserved2 Reserved for future use.
- * @param[out] phContext Returned Application Context.
- *
- * @return Connection status.
- * @retval SCARD_S_SUCCESS Successful (\ref SCARD_S_SUCCESS)
- * @retval SCARD_E_INVALID_VALUE Invalid scope type passed (\ref SCARD_E_INVALID_VALUE)
- * @retval SCARD_E_INVALID_PARAMETER phContext is null (\ref SCARD_E_INVALID_PARAMETER)
- */
 LONG SCardEstablishContext(DWORD dwScope, /*@unused@*/ LPCVOID pvReserved1,
 	/*@unused@*/ LPCVOID pvReserved2, LPSCARDCONTEXT phContext)
 {
 	(void)pvReserved1;
 	(void)pvReserved2;
-	/*
-	 * Check for NULL pointer
-	 */
-	if (phContext == 0)
-		return SCARD_E_INVALID_PARAMETER;
 
 	if (dwScope != SCARD_SCOPE_USER && dwScope != SCARD_SCOPE_TERMINAL &&
 		dwScope != SCARD_SCOPE_SYSTEM && dwScope != SCARD_SCOPE_GLOBAL)
@@ -231,37 +206,18 @@ LONG SCardReleaseContext(SCARDCONTEXT hContext)
 	return SCARD_S_SUCCESS;
 }
 
-LONG SCardSetTimeout(/*@unused@*/ SCARDCONTEXT hContext,
-	/*@unused@*/ DWORD dwTimeout)
-{
-	/*
-	 * This is only used at the client side of an RPC call but just in
-	 * case someone calls it here
-	 */
-
-	(void)hContext;
-	(void)dwTimeout;
-	return SCARD_E_UNSUPPORTED_FEATURE;
-}
-
 LONG SCardConnect(/*@unused@*/ SCARDCONTEXT hContext, LPCSTR szReader,
 	DWORD dwShareMode, DWORD dwPreferredProtocols, LPSCARDHANDLE phCard,
 	LPDWORD pdwActiveProtocol)
 {
 	LONG rv;
 	READER_CONTEXT * rContext = NULL;
-	DWORD dwStatus;
+	uint32_t readerState;
 
 	(void)hContext;
 	PROFILE_START
 
-	/*
-	 * Check for NULL parameters
-	 */
-	if (szReader == NULL || phCard == NULL || pdwActiveProtocol == NULL)
-		return SCARD_E_INVALID_PARAMETER;
-	else
-		*phCard = 0;
+	*phCard = 0;
 
 	if ((dwShareMode != SCARD_SHARE_DIRECT) &&
 			!(dwPreferredProtocols & SCARD_PROTOCOL_T0) &&
@@ -302,7 +258,7 @@ LONG SCardConnect(/*@unused@*/ SCARDCONTEXT hContext, LPCSTR szReader,
 	/*
 	 * Connect if not exclusive mode
 	 */
-	if (rContext->contexts == SCARD_EXCLUSIVE_CONTEXT)
+	if (rContext->contexts == PCSCLITE_SHARING_EXCLUSIVE_CONTEXT)
 	{
 		Log1(PCSC_LOG_ERROR, "Error Reader Exclusive");
 		return SCARD_E_SHARING_VIOLATION;
@@ -319,33 +275,59 @@ LONG SCardConnect(/*@unused@*/ SCARDCONTEXT hContext, LPCSTR szReader,
 		Log1(PCSC_LOG_INFO, "Lock released");
 	}
 
-	/* the reader has been removed while we were waiting */
-	if (NULL == rContext->readerState)
-		return SCARD_E_NO_SMARTCARD;
-
 	/*******************************************
 	 *
 	 * This section tries to determine the
 	 * presence of a card or not
 	 *
 	 *******************************************/
-	dwStatus = rContext->readerState->readerState;
+	readerState = rContext->readerState->readerState;
 
 	if (dwShareMode != SCARD_SHARE_DIRECT)
 	{
-		if (!(dwStatus & SCARD_PRESENT))
+		if (!(readerState & SCARD_PRESENT))
 		{
 			Log1(PCSC_LOG_ERROR, "Card Not Inserted");
 			return SCARD_E_NO_SMARTCARD;
 		}
 
-		if (dwStatus & SCARD_SWALLOWED)
+		/* Power on (again) the card if needed */
+		(void)pthread_mutex_lock(&rContext->powerState_lock);
+		if (POWER_STATE_UNPOWERED == rContext->powerState)
+		{
+			DWORD dwAtrLen;
+
+			dwAtrLen = sizeof(rContext->readerState->cardAtr);
+			rv = IFDPowerICC(rContext, IFD_POWER_UP,
+				rContext->readerState->cardAtr, &dwAtrLen);
+			rContext->readerState->cardAtrLength = dwAtrLen;
+
+			if (rv == IFD_SUCCESS)
+			{
+				readerState = SCARD_PRESENT | SCARD_POWERED | SCARD_NEGOTIABLE;
+
+				Log1(PCSC_LOG_DEBUG, "power up complete.");
+				LogXxd(PCSC_LOG_DEBUG, "Card ATR: ",
+					rContext->readerState->cardAtr,
+					rContext->readerState->cardAtrLength);
+			}
+			else
+				Log3(PCSC_LOG_ERROR, "Error powering up card: %d 0x%04X",
+					rv, rv);
+		}
+
+		if (! (readerState & SCARD_POWERED))
 		{
 			Log1(PCSC_LOG_ERROR, "Card Not Powered");
+			(void)pthread_mutex_unlock(&rContext->powerState_lock);
 			return SCARD_W_UNPOWERED_CARD;
 		}
-	}
 
+		/* the card is now in use */
+		rContext->powerState = POWER_STATE_INUSE;
+		Log1(PCSC_LOG_DEBUG, "powerState: POWER_STATE_INUSE");
+		(void)pthread_mutex_unlock(&rContext->powerState_lock);
+	}
 
 	/*******************************************
 	 *
@@ -452,10 +434,10 @@ LONG SCardConnect(/*@unused@*/ SCARDCONTEXT hContext, LPCSTR szReader,
 
 	if (dwShareMode == SCARD_SHARE_EXCLUSIVE)
 	{
-		if (rContext->contexts == SCARD_NO_CONTEXT)
+		if (rContext->contexts == PCSCLITE_SHARING_NO_CONTEXT)
 		{
-			rContext->contexts = SCARD_EXCLUSIVE_CONTEXT;
-			(void)RFLockSharing(*phCard);
+			rContext->contexts = PCSCLITE_SHARING_EXCLUSIVE_CONTEXT;
+			(void)RFLockSharing(*phCard, rContext);
 		}
 		else
 		{
@@ -483,10 +465,10 @@ LONG SCardConnect(/*@unused@*/ SCARDCONTEXT hContext, LPCSTR szReader,
 		 * Clean up - there is no more room
 		 */
 		(void)RFDestroyReaderHandle(*phCard);
-		if (rContext->contexts == SCARD_EXCLUSIVE_CONTEXT)
-			rContext->contexts = SCARD_NO_CONTEXT;
+		if (rContext->contexts == PCSCLITE_SHARING_EXCLUSIVE_CONTEXT)
+			rContext->contexts = PCSCLITE_SHARING_NO_CONTEXT;
 		else
-			if (rContext->contexts > SCARD_NO_CONTEXT)
+			if (rContext->contexts > PCSCLITE_SHARING_NO_CONTEXT)
 				rContext->contexts -= 1;
 
 		*phCard = 0;
@@ -538,9 +520,7 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 			!(dwPreferredProtocols & SCARD_PROTOCOL_ANY_OLD))
 		return SCARD_E_PROTO_MISMATCH;
 
-	if (pdwActiveProtocol == NULL)
-		return SCARD_E_INVALID_PARAMETER;
-
+	/* get rContext corresponding to hCard */
 	rv = RFReaderInfoById(hCard, &rContext);
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
@@ -559,7 +539,7 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 	/*
 	 * Make sure no one has a lock on this reader
 	 */
-	rv = RFCheckSharing(hCard);
+	rv = RFCheckSharing(hCard, rContext);
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
@@ -576,19 +556,15 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 		/*
 		 * Currently pcsc-lite keeps the card powered constantly
 		 */
-		dwAtrLen = rContext->readerState->cardAtrLength;
+		dwAtrLen = sizeof(rContext->readerState->cardAtr);
 		if (SCARD_RESET_CARD == dwInitialization)
 			rv = IFDPowerICC(rContext, IFD_RESET,
-				rContext->readerState->cardAtr,
-				&dwAtrLen);
+				rContext->readerState->cardAtr, &dwAtrLen);
 		else
 		{
-			rv = IFDPowerICC(rContext, IFD_POWER_DOWN,
-				rContext->readerState->cardAtr,
-				&dwAtrLen);
+			rv = IFDPowerICC(rContext, IFD_POWER_DOWN, NULL, NULL);
 			rv = IFDPowerICC(rContext, IFD_POWER_UP,
-				rContext->readerState->cardAtr,
-				&dwAtrLen);
+				rContext->readerState->cardAtr, &dwAtrLen);
 		}
 		rContext->readerState->cardAtrLength = dwAtrLen;
 
@@ -596,32 +572,14 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 		rContext->readerState->cardProtocol = SCARD_PROTOCOL_UNDEFINED;
 
 		/*
-		 * Set up the status bit masks on dwStatus
+		 * Set up the status bit masks on readerState
 		 */
 		if (rv == SCARD_S_SUCCESS)
 		{
-			rContext->readerState->readerState |= SCARD_PRESENT;
-			rContext->readerState->readerState &= ~SCARD_ABSENT;
-			rContext->readerState->readerState |= SCARD_POWERED;
-			rContext->readerState->readerState |= SCARD_NEGOTIABLE;
-			rContext->readerState->readerState &= ~SCARD_SPECIFIC;
-			rContext->readerState->readerState &= ~SCARD_SWALLOWED;
-			rContext->readerState->readerState &= ~SCARD_UNKNOWN;
-		}
-		else
-		{
-			rContext->readerState->readerState |= SCARD_PRESENT;
-			rContext->readerState->readerState &= ~SCARD_ABSENT;
-			rContext->readerState->readerState |= SCARD_SWALLOWED;
-			rContext->readerState->readerState &= ~SCARD_POWERED;
-			rContext->readerState->readerState &= ~SCARD_NEGOTIABLE;
-			rContext->readerState->readerState &= ~SCARD_SPECIFIC;
-			rContext->readerState->readerState &= ~SCARD_UNKNOWN;
-			rContext->readerState->cardAtrLength = 0;
-		}
+			rContext->readerState->cardAtrLength = dwAtrLen;
+			rContext->readerState->readerState =
+				SCARD_PRESENT | SCARD_POWERED | SCARD_NEGOTIABLE;
 
-		if (rContext->readerState->cardAtrLength > 0)
-		{
 			Log1(PCSC_LOG_DEBUG, "Reset complete.");
 			LogXxd(PCSC_LOG_DEBUG, "Card ATR: ",
 				rContext->readerState->cardAtr,
@@ -629,23 +587,33 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 		}
 		else
 		{
-			DWORD dwStatus, dwAtrLen2;
-			UCHAR ucAtr[MAX_ATR_SIZE];
-
+			rContext->readerState->cardAtrLength = 0;
 			Log1(PCSC_LOG_ERROR, "Error resetting card.");
-			(void)IFDStatusICC(rContext, &dwStatus, ucAtr, &dwAtrLen2);
-			if (dwStatus & SCARD_PRESENT)
-				return SCARD_W_UNRESPONSIVE_CARD;
-			else
+
+			if (rv == SCARD_W_REMOVED_CARD)
+			{
+				rContext->readerState->readerState = SCARD_ABSENT;
 				return SCARD_E_NO_SMARTCARD;
+			}
+			else
+			{
+				rContext->readerState->readerState =
+					SCARD_PRESENT | SCARD_SWALLOWED;
+				return SCARD_W_UNRESPONSIVE_CARD;
+			}
 		}
 	}
 	else
 		if (dwInitialization == SCARD_LEAVE_CARD)
 		{
-			/*
-			 * Do nothing
-			 */
+			uint32_t readerState = rContext->readerState->readerState;
+
+			if (readerState & SCARD_ABSENT)
+				return SCARD_E_NO_SMARTCARD;
+
+			if ((readerState & SCARD_PRESENT)
+				&& (readerState & SCARD_SWALLOWED))
+				return SCARD_W_UNRESPONSIVE_CARD;
 		}
 
 	/*******************************************
@@ -737,17 +705,17 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 
 	if (dwShareMode == SCARD_SHARE_EXCLUSIVE)
 	{
-		if (rContext->contexts == SCARD_EXCLUSIVE_CONTEXT)
+		if (rContext->contexts == PCSCLITE_SHARING_EXCLUSIVE_CONTEXT)
 		{
 			/*
 			 * Do nothing - we are already exclusive
 			 */
 		} else
 		{
-			if (rContext->contexts == SCARD_LAST_CONTEXT)
+			if (rContext->contexts == PCSCLITE_SHARING_LAST_CONTEXT)
 			{
-				rContext->contexts = SCARD_EXCLUSIVE_CONTEXT;
-				(void)RFLockSharing(hCard);
+				rContext->contexts = PCSCLITE_SHARING_EXCLUSIVE_CONTEXT;
+				(void)RFLockSharing(hCard, rContext);
 			} else
 			{
 				return SCARD_E_SHARING_VIOLATION;
@@ -755,7 +723,7 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 		}
 	} else if (dwShareMode == SCARD_SHARE_SHARED)
 	{
-		if (rContext->contexts != SCARD_EXCLUSIVE_CONTEXT)
+		if (rContext->contexts != PCSCLITE_SHARING_EXCLUSIVE_CONTEXT)
 		{
 			/*
 			 * Do nothing - in sharing mode already
@@ -765,12 +733,12 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 			/*
 			 * We are in exclusive mode but want to share now
 			 */
-			(void)RFUnlockSharing(hCard);
-			rContext->contexts = SCARD_LAST_CONTEXT;
+			(void)RFUnlockSharing(hCard, rContext);
+			rContext->contexts = PCSCLITE_SHARING_LAST_CONTEXT;
 		}
 	} else if (dwShareMode == SCARD_SHARE_DIRECT)
 	{
-		if (rContext->contexts != SCARD_EXCLUSIVE_CONTEXT)
+		if (rContext->contexts != PCSCLITE_SHARING_EXCLUSIVE_CONTEXT)
 		{
 			/*
 			 * Do nothing - in sharing mode already
@@ -780,8 +748,8 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 			/*
 			 * We are in exclusive mode but want to share now
 			 */
-			(void)RFUnlockSharing(hCard);
-			rContext->contexts = SCARD_LAST_CONTEXT;
+			(void)RFUnlockSharing(hCard, rContext);
+			rContext->contexts = PCSCLITE_SHARING_LAST_CONTEXT;
 		}
 	} else
 		return SCARD_E_INVALID_VALUE;
@@ -803,11 +771,11 @@ LONG SCardDisconnect(SCARDHANDLE hCard, DWORD dwDisposition)
 {
 	LONG rv;
 	READER_CONTEXT * rContext = NULL;
-	DWORD dwAtrLen;
 
 	if (hCard == 0)
 		return SCARD_E_INVALID_HANDLE;
 
+	/* get rContext corresponding to hCard */
 	rv = RFReaderInfoById(hCard, &rContext);
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
@@ -834,22 +802,21 @@ LONG SCardDisconnect(SCARDHANDLE hCard, DWORD dwDisposition)
 		Log1(PCSC_LOG_INFO, "Lock released");
 	}
 
-	/* the reader has been removed while we were waiting */
-	if (NULL == rContext->readerState)
-		return SCARD_E_NO_SMARTCARD;
-
 	/*
 	 * Unlock any blocks on this context
 	 */
-	rv = RFUnlockAllSharing(hCard);
+	rv = RFUnlockAllSharing(hCard, rContext);
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
 	Log2(PCSC_LOG_DEBUG, "Active Contexts: %d", rContext->contexts);
+	Log2(PCSC_LOG_DEBUG, "dwDisposition: %d", dwDisposition);
 
 	if (dwDisposition == SCARD_RESET_CARD ||
 		dwDisposition == SCARD_UNPOWER_CARD)
 	{
+		DWORD dwAtrLen;
+
 		/*
 		 * Notify the card has been reset
 		 */
@@ -857,58 +824,75 @@ LONG SCardDisconnect(SCARDHANDLE hCard, DWORD dwDisposition)
 
 		/*
 		 * Currently pcsc-lite keeps the card powered constantly
+		 * unless DISABLE_AUTO_POWER_ON is defined
 		 */
-		dwAtrLen = rContext->readerState->cardAtrLength;
+		dwAtrLen = sizeof(rContext->readerState->cardAtr);
 		if (SCARD_RESET_CARD == dwDisposition)
 			rv = IFDPowerICC(rContext, IFD_RESET,
-				rContext->readerState->cardAtr,
-				&dwAtrLen);
+				rContext->readerState->cardAtr, &dwAtrLen);
 		else
 		{
-			rv = IFDPowerICC(rContext, IFD_POWER_DOWN,
-				rContext->readerState->cardAtr,
-				&dwAtrLen);
+			rv = IFDPowerICC(rContext, IFD_POWER_DOWN, NULL, NULL);
+
+#ifdef DISABLE_AUTO_POWER_ON
+			rContext->powerState = POWER_STATE_UNPOWERED;
+			Log1(PCSC_LOG_DEBUG, "powerState: POWER_STATE_UNPOWERED");
+#else
 			rv = IFDPowerICC(rContext, IFD_POWER_UP,
-				rContext->readerState->cardAtr,
-				&dwAtrLen);
+				rContext->readerState->cardAtr, &dwAtrLen);
+#endif
 		}
-		rContext->readerState->cardAtrLength = dwAtrLen;
 
 		/* the protocol is unset after a power on */
 		rContext->readerState->cardProtocol = SCARD_PROTOCOL_UNDEFINED;
 
-		/*
-		 * Set up the status bit masks on dwStatus
-		 */
-		if (rv == SCARD_S_SUCCESS)
+#ifdef DISABLE_AUTO_POWER_ON
+		if (SCARD_UNPOWER_CARD == dwDisposition)
 		{
-			rContext->readerState->readerState |= SCARD_PRESENT;
-			rContext->readerState->readerState &= ~SCARD_ABSENT;
-			rContext->readerState->readerState |= SCARD_POWERED;
-			rContext->readerState->readerState |= SCARD_NEGOTIABLE;
-			rContext->readerState->readerState &= ~SCARD_SPECIFIC;
-			rContext->readerState->readerState &= ~SCARD_SWALLOWED;
-			rContext->readerState->readerState &= ~SCARD_UNKNOWN;
-		}
-		else
-		{
-			if (rContext->readerState->readerState & SCARD_ABSENT)
-				rContext->readerState->readerState &= ~SCARD_PRESENT;
-			else
-				rContext->readerState->readerState |= SCARD_PRESENT;
-			/* SCARD_ABSENT flag is already set */
-			rContext->readerState->readerState |= SCARD_SWALLOWED;
-			rContext->readerState->readerState &= ~SCARD_POWERED;
-			rContext->readerState->readerState &= ~SCARD_NEGOTIABLE;
-			rContext->readerState->readerState &= ~SCARD_SPECIFIC;
-			rContext->readerState->readerState &= ~SCARD_UNKNOWN;
 			rContext->readerState->cardAtrLength = 0;
+			if (rv == SCARD_S_SUCCESS)
+				rContext->readerState->readerState = SCARD_PRESENT;
+			else
+			{
+				Log3(PCSC_LOG_ERROR, "Error powering down card: %d 0x%04X",
+					rv, rv);
+				if (rv == SCARD_W_REMOVED_CARD)
+					rContext->readerState->readerState = SCARD_ABSENT;
+				else
+					rContext->readerState->readerState =
+						SCARD_PRESENT | SCARD_SWALLOWED;
+			}
+			Log1(PCSC_LOG_INFO, "Skip card power on");
 		}
-
-		if (rContext->readerState->cardAtrLength > 0)
-			Log1(PCSC_LOG_DEBUG, "Reset complete.");
 		else
-			Log1(PCSC_LOG_ERROR, "Error resetting card.");
+#endif
+		{
+			/*
+			 * Set up the status bit masks on readerState
+			 */
+			if (rv == SCARD_S_SUCCESS)
+			{
+				rContext->readerState->cardAtrLength = dwAtrLen;
+				rContext->readerState->readerState =
+					SCARD_PRESENT | SCARD_POWERED | SCARD_NEGOTIABLE;
+
+				Log1(PCSC_LOG_DEBUG, "Reset complete.");
+				LogXxd(PCSC_LOG_DEBUG, "Card ATR: ",
+					rContext->readerState->cardAtr,
+					rContext->readerState->cardAtrLength);
+			}
+			else
+			{
+				rContext->readerState->cardAtrLength = 0;
+				Log1(PCSC_LOG_ERROR, "Error resetting card.");
+
+				if (rv == SCARD_W_REMOVED_CARD)
+					rContext->readerState->readerState = SCARD_ABSENT;
+				else
+					rContext->readerState->readerState =
+						SCARD_PRESENT | SCARD_SWALLOWED;
+			}
+		}
 	}
 	else if (dwDisposition == SCARD_EJECT_CARD)
 	{
@@ -960,8 +944,8 @@ LONG SCardDisconnect(SCARDHANDLE hCard, DWORD dwDisposition)
 	/*
 	 * For exclusive connection reset it to no connections
 	 */
-	if (rContext->contexts == SCARD_EXCLUSIVE_CONTEXT)
-		rContext->contexts = SCARD_NO_CONTEXT;
+	if (rContext->contexts == PCSCLITE_SHARING_EXCLUSIVE_CONTEXT)
+		rContext->contexts = PCSCLITE_SHARING_NO_CONTEXT;
 	else
 	{
 		/*
@@ -973,6 +957,40 @@ LONG SCardDisconnect(SCARDHANDLE hCard, DWORD dwDisposition)
 			rContext->contexts = 0;
 	}
 
+	if (PCSCLITE_SHARING_NO_CONTEXT == rContext->contexts)
+	{
+		RESPONSECODE (*fct)(DWORD) = NULL;
+		DWORD dwGetSize;
+
+		(void)pthread_mutex_lock(&rContext->powerState_lock);
+		if (POWER_STATE_INUSE == rContext->powerState)
+		{
+#ifdef DISABLE_AUTO_POWER_ON
+			if (SCARD_RESET_CARD == dwDisposition)
+			{
+				rContext->powerState = POWER_STATE_GRACE_PERIOD;
+				Log1(PCSC_LOG_DEBUG, "powerState: POWER_STATE_GRACE_PERIOD");
+			}
+#else
+			rContext->powerState = POWER_STATE_GRACE_PERIOD;
+			Log1(PCSC_LOG_DEBUG, "powerState: POWER_STATE_GRACE_PERIOD");
+#endif
+		}
+		(void)pthread_mutex_unlock(&rContext->powerState_lock);
+
+		/* ask to stop the "polling" thread so it can be restarted using
+		 * the correct timeout */
+		dwGetSize = sizeof(fct);
+		rv = IFDGetCapabilities(rContext, TAG_IFD_STOP_POLLING_THREAD,
+			&dwGetSize, (PUCHAR)&fct);
+
+		if ((IFD_SUCCESS == rv) && (dwGetSize == sizeof(fct)))
+		{
+			Log1(PCSC_LOG_INFO, "Stoping polling thread");
+			fct(rContext->slot);
+		}
+	}
+
 	/*
 	 * Propagate new state to reader state
 	 */
@@ -989,11 +1007,8 @@ LONG SCardBeginTransaction(SCARDHANDLE hCard)
 	if (hCard == 0)
 		return SCARD_E_INVALID_HANDLE;
 
+	/* get rContext corresponding to hCard */
 	rv = RFReaderInfoById(hCard, &rContext);
-
-	/*
-	 * Cannot find the hCard in this context
-	 */
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
@@ -1011,10 +1026,11 @@ LONG SCardBeginTransaction(SCARDHANDLE hCard)
 	/*
 	 * Make sure some event has not occurred
 	 */
-	if ((rv = RFCheckReaderEventState(rContext, hCard)) != SCARD_S_SUCCESS)
+	rv = RFCheckReaderEventState(rContext, hCard);
+	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
-	rv = RFLockSharing(hCard);
+	rv = RFLockSharing(hCard, rContext);
 
 	/* if the transaction is not yet ready we sleep a bit so the client
 	 * do not retry immediately */
@@ -1030,7 +1046,6 @@ LONG SCardEndTransaction(SCARDHANDLE hCard, DWORD dwDisposition)
 {
 	LONG rv;
 	READER_CONTEXT * rContext = NULL;
-	DWORD dwAtrLen;
 
 	/*
 	 * Ignoring dwDisposition for now
@@ -1044,11 +1059,8 @@ LONG SCardEndTransaction(SCARDHANDLE hCard, DWORD dwDisposition)
 		&& (dwDisposition != SCARD_EJECT_CARD))
 	return SCARD_E_INVALID_VALUE;
 
+	/* get rContext corresponding to hCard */
 	rv = RFReaderInfoById(hCard, &rContext);
-
-	/*
-	 * Cannot find the hCard in this context
-	 */
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
@@ -1059,30 +1071,28 @@ LONG SCardEndTransaction(SCARDHANDLE hCard, DWORD dwDisposition)
 	/*
 	 * Make sure some event has not occurred
 	 */
-	if ((rv = RFCheckReaderEventState(rContext, hCard)) != SCARD_S_SUCCESS)
+	rv = RFCheckReaderEventState(rContext, hCard);
+	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
 	if (dwDisposition == SCARD_RESET_CARD ||
 		dwDisposition == SCARD_UNPOWER_CARD)
 	{
+		DWORD dwAtrLen;
+
 		/*
 		 * Currently pcsc-lite keeps the card always powered
 		 */
-		dwAtrLen = rContext->readerState->cardAtrLength;
+		dwAtrLen = sizeof(rContext->readerState->cardAtr);
 		if (SCARD_RESET_CARD == dwDisposition)
 			rv = IFDPowerICC(rContext, IFD_RESET,
-				rContext->readerState->cardAtr,
-				&dwAtrLen);
+				rContext->readerState->cardAtr, &dwAtrLen);
 		else
 		{
-			rv = IFDPowerICC(rContext, IFD_POWER_DOWN,
-				rContext->readerState->cardAtr,
-				&dwAtrLen);
+			rv = IFDPowerICC(rContext, IFD_POWER_DOWN, NULL, NULL);
 			rv = IFDPowerICC(rContext, IFD_POWER_UP,
-				rContext->readerState->cardAtr,
-				&dwAtrLen);
+				rContext->readerState->cardAtr, &dwAtrLen);
 		}
-		rContext->readerState->cardAtrLength = dwAtrLen;
 
 		/* the protocol is unset after a power on */
 		rContext->readerState->cardProtocol = SCARD_PROTOCOL_UNDEFINED;
@@ -1093,38 +1103,30 @@ LONG SCardEndTransaction(SCARDHANDLE hCard, DWORD dwDisposition)
 		(void)RFSetReaderEventState(rContext, SCARD_RESET);
 
 		/*
-		 * Set up the status bit masks on dwStatus
+		 * Set up the status bit masks on readerState
 		 */
 		if (rv == SCARD_S_SUCCESS)
 		{
-			rContext->readerState->readerState |= SCARD_PRESENT;
-			rContext->readerState->readerState &= ~SCARD_ABSENT;
-			rContext->readerState->readerState |= SCARD_POWERED;
-			rContext->readerState->readerState |= SCARD_NEGOTIABLE;
-			rContext->readerState->readerState &= ~SCARD_SPECIFIC;
-			rContext->readerState->readerState &= ~SCARD_SWALLOWED;
-			rContext->readerState->readerState &= ~SCARD_UNKNOWN;
+			rContext->readerState->cardAtrLength = dwAtrLen;
+			rContext->readerState->readerState =
+				SCARD_PRESENT | SCARD_POWERED | SCARD_NEGOTIABLE;
+
+			Log1(PCSC_LOG_DEBUG, "Reset complete.");
+			LogXxd(PCSC_LOG_DEBUG, "Card ATR: ",
+				rContext->readerState->cardAtr,
+				rContext->readerState->cardAtrLength);
 		}
 		else
 		{
-			if (rContext->readerState->readerState & SCARD_ABSENT)
-				rContext->readerState->readerState &= ~SCARD_PRESENT;
-			else
-				rContext->readerState->readerState |= SCARD_PRESENT;
-			/* SCARD_ABSENT flag is already set */
-			rContext->readerState->readerState |= SCARD_SWALLOWED;
-			rContext->readerState->readerState &= ~SCARD_POWERED;
-			rContext->readerState->readerState &= ~SCARD_NEGOTIABLE;
-			rContext->readerState->readerState &= ~SCARD_SPECIFIC;
-			rContext->readerState->readerState &= ~SCARD_UNKNOWN;
 			rContext->readerState->cardAtrLength = 0;
-		}
-
-		if (rContext->readerState->cardAtrLength > 0)
-			Log1(PCSC_LOG_DEBUG, "Reset complete.");
-		else
 			Log1(PCSC_LOG_ERROR, "Error resetting card.");
 
+			if (rv == SCARD_W_REMOVED_CARD)
+				rContext->readerState->readerState = SCARD_ABSENT;
+			else
+				rContext->readerState->readerState =
+					SCARD_PRESENT | SCARD_SWALLOWED;
+		}
 	}
 	else if (dwDisposition == SCARD_EJECT_CARD)
 	{
@@ -1170,43 +1172,7 @@ LONG SCardEndTransaction(SCARDHANDLE hCard, DWORD dwDisposition)
 	/*
 	 * Unlock any blocks on this context
 	 */
-	(void)RFUnlockSharing(hCard);
-
-	Log2(PCSC_LOG_DEBUG, "Status: 0x%08X", rv);
-
-	return rv;
-}
-
-LONG SCardCancelTransaction(SCARDHANDLE hCard)
-{
-	LONG rv;
-	READER_CONTEXT * rContext = NULL;
-
-	/*
-	 * Ignoring dwDisposition for now
-	 */
-	if (hCard == 0)
-		return SCARD_E_INVALID_HANDLE;
-
-	rv = RFReaderInfoById(hCard, &rContext);
-
-	/*
-	 * Cannot find the hCard in this context
-	 */
-	if (rv != SCARD_S_SUCCESS)
-		return rv;
-
-	rv = RFFindReaderHandle(hCard);
-	if (rv != SCARD_S_SUCCESS)
-		return rv;
-
-	/*
-	 * Make sure some event has not occurred
-	 */
-	if ((rv = RFCheckReaderEventState(rContext, hCard)) != SCARD_S_SUCCESS)
-		return rv;
-
-	rv = RFUnlockSharing(hCard);
+	(void)RFUnlockSharing(hCard, rContext);
 
 	Log2(PCSC_LOG_DEBUG, "Status: 0x%08X", rv);
 
@@ -1220,25 +1186,37 @@ LONG SCardStatus(SCARDHANDLE hCard, LPSTR mszReaderNames,
 	LONG rv;
 	READER_CONTEXT * rContext = NULL;
 
+	/* These parameters are not used by the client
+	 * Client side code uses readerStates[] instead */
+	(void)mszReaderNames;
+	(void)pcchReaderLen;
+	(void)pdwState;
+	(void)pdwProtocol;
+	(void)pbAtr;
+	(void)pcbAtrLen;
+
 	if (hCard == 0)
 		return SCARD_E_INVALID_HANDLE;
 
+	/* get rContext corresponding to hCard */
+	rv = RFReaderInfoById(hCard, &rContext);
+	if (rv != SCARD_S_SUCCESS)
+		return rv;
+
 	/*
 	 * Make sure no one has a lock on this reader
 	 */
-	if ((rv = RFCheckSharing(hCard)) != SCARD_S_SUCCESS)
+	rv = RFCheckSharing(hCard, rContext);
+	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
-	rv = RFReaderInfoById(hCard, &rContext);
-
 	/*
 	 * Cannot find the hCard in this context
 	 */
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
-	if (strlen(rContext->lpcReader) > MAX_BUFFER_SIZE
-			|| rContext->readerState->cardAtrLength > MAX_ATR_SIZE)
+	if (rContext->readerState->cardAtrLength > MAX_ATR_SIZE)
 		return SCARD_F_INTERNAL_ERROR;
 
 	/*
@@ -1250,7 +1228,8 @@ LONG SCardStatus(SCARDHANDLE hCard, LPSTR mszReaderNames,
 	/*
 	 * Make sure some event has not occurred
 	 */
-	if ((rv = RFCheckReaderEventState(rContext, hCard)) != SCARD_S_SUCCESS)
+	rv = RFCheckReaderEventState(rContext, hCard);
+	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
 	/*
@@ -1260,95 +1239,9 @@ LONG SCardStatus(SCARDHANDLE hCard, LPSTR mszReaderNames,
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
-	if (mszReaderNames)
-	{  /* want reader name */
-		if (pcchReaderLen)
-		{ /* & present reader name length */
-			if (*pcchReaderLen >= strlen(rContext->lpcReader))
-			{ /* & enough room */
-				*pcchReaderLen = strlen(rContext->lpcReader);
-				strncpy(mszReaderNames, rContext->lpcReader, MAX_READERNAME);
-			}
-			else
-			{        /* may report only reader name len */
-				*pcchReaderLen = strlen(rContext->lpcReader);
-				rv = SCARD_E_INSUFFICIENT_BUFFER;
-			}
-		}
-		else
-		{            /* present buf & no buflen */
-			return SCARD_E_INVALID_PARAMETER;
-		}
-	}
-	else
-	{
-		if (pcchReaderLen)
-		{ /* want reader len only */
-			*pcchReaderLen = strlen(rContext->lpcReader);
-		}
-		else
-		{
-		/* nothing todo */
-		}
-	}
-
-	if (pdwState)
-		*pdwState = rContext->readerState->readerState;
-
-	if (pdwProtocol)
-		*pdwProtocol = rContext->readerState->cardProtocol;
-
-	if (pbAtr)
-	{  /* want ATR */
-		if (pcbAtrLen)
-		{ /* & present ATR length */
-			if (*pcbAtrLen >= rContext->readerState->cardAtrLength)
-			{ /* & enough room */
-				*pcbAtrLen = rContext->readerState->cardAtrLength;
-				memcpy(pbAtr, rContext->readerState->cardAtr,
-					rContext->readerState->cardAtrLength);
-			}
-			else
-			{ /* may report only ATR len */
-				*pcbAtrLen = rContext->readerState->cardAtrLength;
-				rv = SCARD_E_INSUFFICIENT_BUFFER;
-			}
-		}
-		else
-		{ /* present buf & no buflen */
-			return SCARD_E_INVALID_PARAMETER;
-		}
-	}
-	else
-	{
-		if (pcbAtrLen)
-		{ /* want ATR len only */
-			*pcbAtrLen = rContext->readerState->cardAtrLength;
-		}
-		else
-		{
-			/* nothing todo */
-		}
-	}
-
 	return rv;
 }
 
-LONG SCardGetStatusChange(/*@unused@*/ SCARDCONTEXT hContext,
-	/*@unused@*/ DWORD dwTimeout,
-	/*@unused@*/ LPSCARD_READERSTATE_A rgReaderStates,
-	/*@unused@*/ DWORD cReaders)
-{
-	/*
-	 * Client side function
-	 */
-	(void)hContext;
-	(void)dwTimeout;
-	(void)rgReaderStates;
-	(void)cReaders;
-	return SCARD_S_SUCCESS;
-}
-
 LONG SCardControl(SCARDHANDLE hCard, DWORD dwControlCode,
 	LPCVOID pbSendBuffer, DWORD cbSendLength,
 	LPVOID pbRecvBuffer, DWORD cbRecvLength, LPDWORD lpBytesReturned)
@@ -1362,13 +1255,15 @@ LONG SCardControl(SCARDHANDLE hCard, DWORD dwControlCode,
 	if (0 == hCard)
 		return SCARD_E_INVALID_HANDLE;
 
+	/* get rContext corresponding to hCard */
+	rv = RFReaderInfoById(hCard, &rContext);
+	if (rv != SCARD_S_SUCCESS)
+		return rv;
+
 	/*
 	 * Make sure no one has a lock on this reader
 	 */
-	if ((rv = RFCheckSharing(hCard)) != SCARD_S_SUCCESS)
-		return rv;
-
-	rv = RFReaderInfoById(hCard, &rContext);
+	rv = RFCheckSharing(hCard, rContext);
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
@@ -1411,13 +1306,15 @@ LONG SCardGetAttrib(SCARDHANDLE hCard, DWORD dwAttrId,
 	if (0 == hCard)
 		return SCARD_E_INVALID_HANDLE;
 
+	/* get rContext corresponding to hCard */
+	rv = RFReaderInfoById(hCard, &rContext);
+	if (rv != SCARD_S_SUCCESS)
+		return rv;
+
 	/*
 	 * Make sure no one has a lock on this reader
 	 */
-	if ((rv = RFCheckSharing(hCard)) != SCARD_S_SUCCESS)
-		return rv;
-
-	rv = RFReaderInfoById(hCard, &rContext);
+	rv = RFCheckSharing(hCard, rContext);
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
@@ -1435,7 +1332,8 @@ LONG SCardGetAttrib(SCARDHANDLE hCard, DWORD dwAttrId,
 	/*
 	 * Make sure some event has not occurred
 	 */
-	if ((rv = RFCheckReaderEventState(rContext, hCard)) != SCARD_S_SUCCESS)
+	rv = RFCheckReaderEventState(rContext, hCard);
+	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
 	rv = IFDGetCapabilities(rContext, dwAttrId, pcbAttrLen, pbAttr);
@@ -1450,20 +1348,20 @@ LONG SCardGetAttrib(SCARDHANDLE hCard, DWORD dwAttrId,
 			 */
 			if (dwAttrId == SCARD_ATTR_DEVICE_FRIENDLY_NAME)
 			{
-				unsigned int len = strlen(rContext->lpcReader)+1;
+				unsigned int len = strlen(rContext->readerState->readerName)+1;
 
 				*pcbAttrLen = len;
 				if (len > *pcbAttrLen)
 					rv = SCARD_E_INSUFFICIENT_BUFFER;
 				else
 				{
-					(void)strlcpy((char *)pbAttr, rContext->lpcReader,
-						*pcbAttrLen);
+					(void)strlcpy((char *)pbAttr,
+						rContext->readerState->readerName, *pcbAttrLen);
 					rv = SCARD_S_SUCCESS;
 				}
 
 			}
-			else 
+			else
 				rv = SCARD_E_UNSUPPORTED_FEATURE;
 			break;
 		case IFD_ERROR_INSUFFICIENT_BUFFER:
@@ -1485,13 +1383,15 @@ LONG SCardSetAttrib(SCARDHANDLE hCard, DWORD dwAttrId,
 	if (0 == hCard)
 		return SCARD_E_INVALID_HANDLE;
 
+	/* get rContext corresponding to hCard */
+	rv = RFReaderInfoById(hCard, &rContext);
+	if (rv != SCARD_S_SUCCESS)
+		return rv;
+
 	/*
 	 * Make sure no one has a lock on this reader
 	 */
-	if ((rv = RFCheckSharing(hCard)) != SCARD_S_SUCCESS)
-		return rv;
-
-	rv = RFReaderInfoById(hCard, &rContext);
+	rv = RFCheckSharing(hCard, rContext);
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
@@ -1509,7 +1409,8 @@ LONG SCardSetAttrib(SCARDHANDLE hCard, DWORD dwAttrId,
 	/*
 	 * Make sure some event has not occurred
 	 */
-	if ((rv = RFCheckReaderEventState(rContext, hCard)) != SCARD_S_SUCCESS)
+	rv = RFCheckReaderEventState(rContext, hCard);
+	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
 	rv = IFDSetCapabilities(rContext, dwAttrId, cbAttrLen, (PUCHAR)pbAttr);
@@ -1522,9 +1423,9 @@ LONG SCardSetAttrib(SCARDHANDLE hCard, DWORD dwAttrId,
 			return SCARD_E_NOT_TRANSACTED;
 }
 
-LONG SCardTransmit(SCARDHANDLE hCard, LPCSCARD_IO_REQUEST pioSendPci,
+LONG SCardTransmit(SCARDHANDLE hCard, const SCARD_IO_REQUEST *pioSendPci,
 	LPCBYTE pbSendBuffer, DWORD cbSendLength,
-	LPSCARD_IO_REQUEST pioRecvPci, LPBYTE pbRecvBuffer,
+	SCARD_IO_REQUEST *pioRecvPci, LPBYTE pbRecvBuffer,
 	LPDWORD pcbRecvLength)
 {
 	LONG rv;
@@ -1532,31 +1433,27 @@ LONG SCardTransmit(SCARDHANDLE hCard, LPCSCARD_IO_REQUEST pioSendPci,
 	SCARD_IO_HEADER sSendPci, sRecvPci;
 	DWORD dwRxLength, tempRxLength;
 
-	if (pcbRecvLength == 0)
-		return SCARD_E_INVALID_PARAMETER;
-
 	dwRxLength = *pcbRecvLength;
 	*pcbRecvLength = 0;
 
 	if (hCard == 0)
 		return SCARD_E_INVALID_HANDLE;
 
-	if (pbSendBuffer == NULL || pbRecvBuffer == NULL || pioSendPci == NULL)
-		return SCARD_E_INVALID_PARAMETER;
-
 	/*
 	 * Must at least have 2 status words even for SCardControl
 	 */
 	if (dwRxLength < 2)
 		return SCARD_E_INSUFFICIENT_BUFFER;
 
+	/* get rContext corresponding to hCard */
+	rv = RFReaderInfoById(hCard, &rContext);
+	if (rv != SCARD_S_SUCCESS)
+		return rv;
+
 	/*
 	 * Make sure no one has a lock on this reader
 	 */
-	if ((rv = RFCheckSharing(hCard)) != SCARD_S_SUCCESS)
-		return rv;
-
-	rv = RFReaderInfoById(hCard, &rContext);
+	rv = RFCheckSharing(hCard, rContext);
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
@@ -1574,7 +1471,8 @@ LONG SCardTransmit(SCARDHANDLE hCard, LPCSCARD_IO_REQUEST pioSendPci,
 	/*
 	 * Make sure some event has not occurred
 	 */
-	if ((rv = RFCheckReaderEventState(rContext, hCard)) != SCARD_S_SUCCESS)
+	rv = RFCheckReaderEventState(rContext, hCard);
+	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
 	/*
@@ -1678,27 +1576,3 @@ LONG SCardTransmit(SCARDHANDLE hCard, LPCSCARD_IO_REQUEST pioSendPci,
 	return SCARD_S_SUCCESS;
 }
 
-LONG SCardListReaders(/*@unused@*/ SCARDCONTEXT hContext,
-	/*@unused@*/ LPCSTR mszGroups,
-	/*@unused@*/ LPSTR mszReaders,
-	/*@unused@*/ LPDWORD pcchReaders)
-{
-	/*
-	 * Client side function
-	 */
-	(void)hContext;
-	(void)mszGroups;
-	(void)mszReaders;
-	(void)pcchReaders;
-	return SCARD_S_SUCCESS;
-}
-
-LONG SCardCancel(/*@unused@*/ SCARDCONTEXT hContext)
-{
-	/*
-	 * Client side function
-	 */
-	(void)hContext;
-	return SCARD_S_SUCCESS;
-}
-
