@@ -5,12 +5,12 @@
  *  David Corcoran <corcoran@linuxnet.com>
  * Copyright (C) 2003-2004
  *  Damien Sauveron <damien.sauveron@labri.fr>
- * Copyright (C) 2002-2009
+ * Copyright (C) 2002-2010
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  * Copyright (C) 2009
  *  Jean-Luc Giraud <jlgiraud@googlemail.com>
  *
- * $Id: winscard_svc.c 4967 2010-05-29 11:28:16Z rousseau $
+ * $Id: winscard_svc.c 5386 2010-11-08 10:00:58Z rousseau $
  */
 
 /**
@@ -61,9 +61,8 @@ struct _psContext
 	int32_t hContext;
 	list_t cardsList;
 	pthread_mutex_t cardsList_lock;	/**< lock for the above list */
-	uint32_t dwClientID;			/**< Connection ID used to reference the Client. */
-	pthread_t pthThread;		/**< Event polling thread's ID */
-	int protocol_major, protocol_minor;	/**< Protocol number agreed between client and server*/
+	uint32_t dwClientID;	/**< Connection ID used to reference the Client. */
+	pthread_t pthThread;	/**< Event polling thread's ID */
 };
 typedef struct _psContext SCONTEXT;
 
@@ -84,7 +83,8 @@ static int contextsListhContext_seeker(const void *el, const void *key)
 
 	if ((el == NULL) || (key == NULL))
 	{
-		Log3(PCSC_LOG_CRITICAL, "contextsListhContext_seeker called with NULL pointer: el=%X, key=%X", el, key);
+		Log3(PCSC_LOG_CRITICAL, "called with NULL pointer: el=%X, key=%X",
+			el, key);
 	}
 
 	if (currentContext->hContext == *(int32_t *)key)
@@ -92,7 +92,8 @@ static int contextsListhContext_seeker(const void *el, const void *key)
 	return 0;
 }
 
-LONG ContextsInitialize(int customMaxThreadCounter, int customMaxThreadCardHandles)
+LONG ContextsInitialize(int customMaxThreadCounter,
+	int customMaxThreadCardHandles)
 {
 	int lrv = 0;
 
@@ -105,13 +106,14 @@ LONG ContextsInitialize(int customMaxThreadCounter, int customMaxThreadCardHandl
 	lrv = list_init(&contextsList);
 	if (lrv < 0)
 	{
-		Log2(PCSC_LOG_CRITICAL, "list_init failed with return value: %X", lrv);
+		Log2(PCSC_LOG_CRITICAL, "list_init failed with return value: %d", lrv);
 		return -1;
 	}
 	lrv = list_attributes_seeker(& contextsList, contextsListhContext_seeker);
 	if (lrv < 0)
 	{
-		Log2(PCSC_LOG_CRITICAL, "list_attributes_seeker failed with return value: %X", lrv);
+		Log2(PCSC_LOG_CRITICAL,
+			"list_attributes_seeker failed with return value: %d", lrv);
 		return -1;
 	}
 
@@ -170,7 +172,7 @@ LONG CreateContextThread(uint32_t *pdwClientID)
 	lrv = list_init(&(newContext->cardsList));
 	if (lrv < 0)
 	{
-		Log2(PCSC_LOG_CRITICAL, "list_init failed with return value: %X", lrv);
+		Log2(PCSC_LOG_CRITICAL, "list_init failed with return value: %d", lrv);
 		goto error;
 	}
 
@@ -182,10 +184,12 @@ LONG CreateContextThread(uint32_t *pdwClientID)
 	 * usefull even on a 64-bit CPU since the API between pcscd and
 	 * libpcscliter uses "int32_t hCard;"
 	 */
-	lrv = list_attributes_comparator(&(newContext->cardsList), list_comparator_int32_t);
+	lrv = list_attributes_comparator(&(newContext->cardsList),
+		list_comparator_int32_t);
 	if (lrv != 0)
 	{
-		Log2(PCSC_LOG_CRITICAL, "list_attributes_comparator failed with return value: %X", lrv);
+		Log2(PCSC_LOG_CRITICAL,
+			"list_attributes_comparator failed with return value: %d", lrv);
 		list_destroy(&(newContext->cardsList));
 		goto error;
 	}
@@ -197,7 +201,8 @@ LONG CreateContextThread(uint32_t *pdwClientID)
 	(void)pthread_mutex_unlock(&contextsList_lock);
 	if (lrv < 0)
 	{
-		Log2(PCSC_LOG_CRITICAL, "list_append failed with return value: %X", lrv);
+		Log2(PCSC_LOG_CRITICAL, "list_append failed with return value: %d",
+			lrv);
 		list_destroy(&(newContext->cardsList));
 		goto error;
 	}
@@ -213,7 +218,7 @@ LONG CreateContextThread(uint32_t *pdwClientID)
 		lrv2 = list_delete(&contextsList, newContext);
 		(void)pthread_mutex_unlock(&contextsList_lock);
 		if (lrv2 < 0)
-			Log2(PCSC_LOG_CRITICAL, "list_delete failed with error %X", lrv2);
+			Log2(PCSC_LOG_CRITICAL, "list_delete failed with error %d", lrv2);
 		list_destroy(&(newContext->cardsList));
 		goto error;
 	}
@@ -274,7 +279,7 @@ static const char *CommandsText[] = {
 #define READ_BODY(v) \
 	if (header.size != sizeof(v)) { goto wrong_length; } \
 	ret = MessageReceive(&v, sizeof(v), filedes); \
-	if (ret < 0) { Log2(PCSC_LOG_DEBUG, "Client die: %d", filedes); goto exit; }
+	if (ret != SCARD_S_SUCCESS) { Log2(PCSC_LOG_DEBUG, "Client die: %d", filedes); goto exit; }
 
 #define WRITE_BODY(v) \
 	WRITE_BODY_WITH_COMMAND(CommandsText[header.command], v)
@@ -295,7 +300,7 @@ static void ContextThread(LPVOID newContext)
 		struct rxHeader header;
 		int32_t ret = MessageReceive(&header, sizeof(header), filedes);
 
-		if (ret < 0)
+		if (ret != SCARD_S_SUCCESS)
 		{
 			/* Clean up the dead client */
 			Log2(PCSC_LOG_DEBUG, "Client die: %d", filedes);
@@ -317,27 +322,19 @@ static void ContextThread(LPVOID newContext)
 
 				READ_BODY(veStr)
 
-				/* get the client protocol version */
-				threadContext->protocol_major = veStr.major;
-				threadContext->protocol_minor = veStr.minor;
-
-				Log3(PCSC_LOG_DEBUG,
-						"Client is protocol version %d:%d",
-						veStr.major, veStr.minor);
+				Log3(PCSC_LOG_DEBUG, "Client is protocol version %d:%d",
+					veStr.major, veStr.minor);
 
 				veStr.rv = SCARD_S_SUCCESS;
 
-				/* client is newer than server */
-				if ((veStr.major > PROTOCOL_VERSION_MAJOR)
-						|| (veStr.major == PROTOCOL_VERSION_MAJOR
-							&& veStr.minor > PROTOCOL_VERSION_MINOR))
+				/* client and server use different protocol */
+				if ((veStr.major != PROTOCOL_VERSION_MAJOR)
+					|| (veStr.minor != PROTOCOL_VERSION_MINOR))
 				{
-					Log3(PCSC_LOG_CRITICAL,
-							"Client protocol is too new %d:%d",
-							veStr.major, veStr.minor);
-					Log3(PCSC_LOG_CRITICAL,
-							"Server protocol is %d:%d",
-							PROTOCOL_VERSION_MAJOR, PROTOCOL_VERSION_MINOR);
+					Log3(PCSC_LOG_CRITICAL, "Client protocol is %d:%d",
+						veStr.major, veStr.minor);
+					Log3(PCSC_LOG_CRITICAL, "Server protocol is %d:%d",
+						PROTOCOL_VERSION_MAJOR, PROTOCOL_VERSION_MINOR);
 					veStr.rv = SCARD_E_NO_SERVICE;
 				}
 
@@ -395,12 +392,12 @@ static void ContextThread(LPVOID newContext)
 				READ_BODY(esStr)
 
 				hContext = esStr.hContext;
-				esStr.rv = SCardEstablishContext(esStr.dwScope, 0, 0, &hContext);
+				esStr.rv = SCardEstablishContext(esStr.dwScope, 0, 0,
+					&hContext);
 				esStr.hContext = hContext;
 
 				if (esStr.rv == SCARD_S_SUCCESS)
-					esStr.rv =
-						MSGAddContext(esStr.hContext, threadContext);
+					esStr.rv = MSGAddContext(esStr.hContext, threadContext);
 
 				WRITE_BODY(esStr)
 			}
@@ -415,8 +412,7 @@ static void ContextThread(LPVOID newContext)
 				reStr.rv = SCardReleaseContext(reStr.hContext);
 
 				if (reStr.rv == SCARD_S_SUCCESS)
-					reStr.rv =
-						MSGRemoveContext(reStr.hContext, threadContext);
+					reStr.rv = MSGRemoveContext(reStr.hContext, threadContext);
 
 				WRITE_BODY(reStr)
 			}
@@ -434,15 +430,15 @@ static void ContextThread(LPVOID newContext)
 				dwActiveProtocol = coStr.dwActiveProtocol;
 
 				coStr.rv = SCardConnect(coStr.hContext, coStr.szReader,
-						coStr.dwShareMode, coStr.dwPreferredProtocols,
-						&hCard, &dwActiveProtocol);
+					coStr.dwShareMode, coStr.dwPreferredProtocols,
+					&hCard, &dwActiveProtocol);
 
 				coStr.hCard = hCard;
 				coStr.dwActiveProtocol = dwActiveProtocol;
 
 				if (coStr.rv == SCARD_S_SUCCESS)
-					coStr.rv =
-						MSGAddHandle(coStr.hContext, coStr.hCard, threadContext);
+					coStr.rv = MSGAddHandle(coStr.hContext, coStr.hCard,
+						threadContext);
 
 				WRITE_BODY(coStr)
 			}
@@ -459,8 +455,8 @@ static void ContextThread(LPVOID newContext)
 					goto exit;
 
 				rcStr.rv = SCardReconnect(rcStr.hCard, rcStr.dwShareMode,
-						rcStr.dwPreferredProtocols,
-						rcStr.dwInitialization, &dwActiveProtocol);
+					rcStr.dwPreferredProtocols, rcStr.dwInitialization,
+					&dwActiveProtocol);
 				rcStr.dwActiveProtocol = dwActiveProtocol;
 
 				WRITE_BODY(rcStr)
@@ -470,19 +466,16 @@ static void ContextThread(LPVOID newContext)
 			case SCARD_DISCONNECT:
 			{
 				struct disconnect_struct diStr;
-				LONG rv;
 
 				READ_BODY(diStr)
 
-				rv = MSGCheckHandleAssociation(diStr.hCard, threadContext);
-				if (0 == rv)
-				{
-					diStr.rv = SCardDisconnect(diStr.hCard, diStr.dwDisposition);
+				if (MSGCheckHandleAssociation(diStr.hCard, threadContext))
+					goto exit;
 
-					if (SCARD_S_SUCCESS == diStr.rv)
-						diStr.rv =
-							MSGRemoveHandle(diStr.hCard, threadContext);
-				}
+				diStr.rv = SCardDisconnect(diStr.hCard, diStr.dwDisposition);
+
+				if (SCARD_S_SUCCESS == diStr.rv)
+					diStr.rv = MSGRemoveHandle(diStr.hCard, threadContext);
 
 				WRITE_BODY(diStr)
 			}
@@ -491,13 +484,13 @@ static void ContextThread(LPVOID newContext)
 			case SCARD_BEGIN_TRANSACTION:
 			{
 				struct begin_struct beStr;
-				LONG rv;
 
 				READ_BODY(beStr)
 
-				rv = MSGCheckHandleAssociation(beStr.hCard, threadContext);
-				if (0 == rv)
-					beStr.rv = SCardBeginTransaction(beStr.hCard);
+				if (MSGCheckHandleAssociation(beStr.hCard, threadContext))
+					goto exit;
+
+				beStr.rv = SCardBeginTransaction(beStr.hCard);
 
 				WRITE_BODY(beStr)
 			}
@@ -506,34 +499,19 @@ static void ContextThread(LPVOID newContext)
 			case SCARD_END_TRANSACTION:
 			{
 				struct end_struct enStr;
-				LONG rv;
 
 				READ_BODY(enStr)
 
-				rv = MSGCheckHandleAssociation(enStr.hCard, threadContext);
-				if (0 == rv)
-					enStr.rv =
-						SCardEndTransaction(enStr.hCard, enStr.dwDisposition);
+				if (MSGCheckHandleAssociation(enStr.hCard, threadContext))
+					goto exit;
+
+				enStr.rv = SCardEndTransaction(enStr.hCard,
+					enStr.dwDisposition);
 
 				WRITE_BODY(enStr)
 			}
 			break;
 
-			case SCARD_CANCEL_TRANSACTION:
-			{
-				struct cancel_transaction_struct caStr;
-				LONG rv;
-
-				READ_BODY(caStr)
-
-				rv = MSGCheckHandleAssociation(caStr.hCard, threadContext);
-				if (0 == rv)
-					caStr.rv = SCardCancelTransaction(caStr.hCard);
-
-				WRITE_BODY(caStr)
-			}
-			break;
-
 			case SCARD_CANCEL:
 			{
 				struct cancel_struct caStr;
@@ -560,42 +538,15 @@ static void ContextThread(LPVOID newContext)
 			case SCARD_STATUS:
 			{
 				struct status_struct stStr;
-				LONG rv;
 
 				READ_BODY(stStr)
 
-				rv = MSGCheckHandleAssociation(stStr.hCard, threadContext);
-				if (0 == rv)
-				{
-					DWORD cchReaderLen;
-					DWORD dwState;
-					DWORD dwProtocol;
-					DWORD cbAtrLen;
+				if (MSGCheckHandleAssociation(stStr.hCard, threadContext))
+					goto exit;
 
-					cchReaderLen = stStr.pcchReaderLen;
-					dwState = stStr.dwState;
-					dwProtocol = stStr.dwProtocol;
-					cbAtrLen = stStr.pcbAtrLen;
-
-					/* avoids buffer overflow */
-					if ((cchReaderLen > sizeof(stStr.mszReaderNames))
-						|| (cbAtrLen > sizeof(stStr.pbAtr)))
-					{
-						stStr.rv = SCARD_E_INSUFFICIENT_BUFFER ;
-					}
-					else
-					{
-						stStr.rv = SCardStatus(stStr.hCard,
-							stStr.mszReaderNames,
-							&cchReaderLen, &dwState,
-							&dwProtocol, stStr.pbAtr, &cbAtrLen);
-
-						stStr.pcchReaderLen = cchReaderLen;
-						stStr.dwState = dwState;
-						stStr.dwProtocol = dwProtocol;
-						stStr.pcbAtrLen = cbAtrLen;
-					}
-				}
+				/* only hCard and return value are used by the client */
+				stStr.rv = SCardStatus(stStr.hCard, NULL, NULL, NULL,
+					NULL, 0, NULL);
 
 				WRITE_BODY(stStr)
 			}
@@ -618,11 +569,11 @@ static void ContextThread(LPVOID newContext)
 				/* avoids buffer overflow */
 				if ((trStr.pcbRecvLength > sizeof(pbRecvBuffer))
 					|| (trStr.cbSendLength > sizeof(pbSendBuffer)))
-					goto exit;
+					goto buffer_overflow;
 
 				/* read sent buffer */
 				ret = MessageReceive(pbSendBuffer, trStr.cbSendLength, filedes);
-				if (ret < 0)
+				if (ret != SCARD_S_SUCCESS)
 				{
 					Log2(PCSC_LOG_DEBUG, "Client die: %d", filedes);
 					goto exit;
@@ -668,12 +619,12 @@ static void ContextThread(LPVOID newContext)
 				if ((ctStr.cbRecvLength > sizeof(pbRecvBuffer))
 					|| (ctStr.cbSendLength > sizeof(pbSendBuffer)))
 				{
-					goto exit;
+					goto buffer_overflow;
 				}
 
 				/* read sent buffer */
 				ret = MessageReceive(pbSendBuffer, ctStr.cbSendLength, filedes);
-				if (ret < 0)
+				if (ret != SCARD_S_SUCCESS)
 				{
 					Log2(PCSC_LOG_DEBUG, "Client die: %d", filedes);
 					goto exit;
@@ -713,7 +664,7 @@ static void ContextThread(LPVOID newContext)
 				cbAttrLen = gsStr.cbAttrLen;
 
 				gsStr.rv = SCardGetAttrib(gsStr.hCard, gsStr.dwAttrId,
-						gsStr.pbAttr, &cbAttrLen);
+					gsStr.pbAttr, &cbAttrLen);
 
 				gsStr.cbAttrLen = cbAttrLen;
 
@@ -728,7 +679,7 @@ static void ContextThread(LPVOID newContext)
 				READ_BODY(gsStr)
 
 				if (MSGCheckHandleAssociation(gsStr.hCard, threadContext))
-					goto buffer_overflow;
+					goto exit;
 
 				/* avoids buffer overflow */
 				if (gsStr.cbAttrLen > sizeof(gsStr.pbAttr))
@@ -747,7 +698,7 @@ static void ContextThread(LPVOID newContext)
 		}
 
 		/* MessageSend() failed */
-		if (-1 == ret)
+		if (ret != SCARD_S_SUCCESS)
 		{
 			/* Clean up the dead client */
 			Log2(PCSC_LOG_DEBUG, "Client die: %d", filedes);
@@ -851,7 +802,7 @@ static LONG MSGRemoveContext(SCARDCONTEXT hContext, SCONTEXT * threadContext)
 		lrv = list_delete_at(&(threadContext->cardsList), 0);
 		if (lrv < 0)
 			Log2(PCSC_LOG_CRITICAL,
-				"list_delete_at failed with return value: %X", lrv);
+				"list_delete_at failed with return value: %d", lrv);
 	}
 	(void)pthread_mutex_unlock(&threadContext->cardsList_lock);
 	list_destroy(&(threadContext->cardsList));
@@ -876,7 +827,8 @@ static LONG MSGAddHandle(SCARDCONTEXT hContext, SCARDHANDLE hCard,
 		listLength = list_size(&(threadContext->cardsList));
 		if (listLength >= contextMaxCardHandles)
 		{
-			Log4(PCSC_LOG_DEBUG, "Too many card handles for thread context @%X: %d (max is %d)"
+			Log4(PCSC_LOG_DEBUG,
+				"Too many card handles for thread context @%X: %d (max is %d)"
 				"Restart pcscd with --max-card-handle-per-thread value",
 				threadContext, listLength, contextMaxCardHandles);
 			return SCARD_E_NO_MEMORY;
@@ -887,7 +839,7 @@ static LONG MSGAddHandle(SCARDCONTEXT hContext, SCARDHANDLE hCard,
 		(void)pthread_mutex_unlock(&threadContext->cardsList_lock);
 		if (lrv < 0)
 		{
-			Log2(PCSC_LOG_CRITICAL, "list_append failed with return value: %X",
+			Log2(PCSC_LOG_CRITICAL, "list_append failed with return value: %d",
 				lrv);
 			return SCARD_E_NO_MEMORY;
 		}
@@ -906,7 +858,7 @@ static LONG MSGRemoveHandle(SCARDHANDLE hCard, SCONTEXT * threadContext)
 	(void)pthread_mutex_unlock(&threadContext->cardsList_lock);
 	if (lrv < 0)
 	{
-		Log2(PCSC_LOG_CRITICAL, "list_delete failed with error %X", lrv);
+		Log2(PCSC_LOG_CRITICAL, "list_delete failed with error %d", lrv);
 		return SCARD_E_INVALID_VALUE;
 	}
 
@@ -914,9 +866,19 @@ static LONG MSGRemoveHandle(SCARDHANDLE hCard, SCONTEXT * threadContext)
 }
 
 
-static LONG MSGCheckHandleAssociation(SCARDHANDLE hCard, SCONTEXT * threadContext)
+static LONG MSGCheckHandleAssociation(SCARDHANDLE hCard,
+	SCONTEXT * threadContext)
 {
 	int list_index = 0;
+
+	if (0 == threadContext->hContext)
+	{
+		/* the handle is no more valid. After SCardReleaseContext() for
+		 * example */
+		Log1(PCSC_LOG_CRITICAL, "Invalidated handle");
+		return -1;
+	}
+
 	(void)pthread_mutex_lock(&threadContext->cardsList_lock);
 	list_index = list_locate(&(threadContext->cardsList), &hCard);
 	(void)pthread_mutex_unlock(&threadContext->cardsList_lock);
